@@ -18,6 +18,7 @@ from .abc import (
     StorageProgressStart,
     StorageProgressStep,
 )
+from .blob_storage import Blob, BlobListing, BlobStorage, BucketListing, PrefixListing
 from .client import Client, Preset
 from .config import Config, ConfigError
 from .config_factory import (
@@ -41,6 +42,7 @@ from .jobs import (
     Container,
     HTTPPort,
     JobDescription,
+    JobRestartPolicy,
     Jobs,
     JobStatus,
     JobStatusHistory,
@@ -63,6 +65,7 @@ __all__ = (
     "CONFIG_ENV_NAME",
     "Jobs",
     "JobDescription",
+    "JobRestartPolicy",
     "JobStatus",
     "JobStatusHistory",
     "JobTelemetry",
@@ -75,6 +78,11 @@ __all__ = (
     "Share",
     "Client",
     "Preset",
+    "BlobStorage",
+    "BucketListing",
+    "BlobListing",
+    "Blob",
+    "PrefixListing",
     "Storage",
     "FileStatusType",
     "FileStatus",
