@@ -154,6 +154,12 @@ class Config(metaclass=NoPublicConstructor):
         cluster = self._config_data.clusters[self._config_data.cluster_name]
         return cluster.monitoring_url
 
+    @property
+    def blob_storage_url(self) -> URL:
+        cluster = self._config_data.clusters[self._config_data.cluster_name]
+        # XXX: Updata properly the API to return Object storage as separate URL
+        return URL(str(cluster.storage_url).replace("/storage", "/blob"))
+
     @property
     def storage_url(self) -> URL:
         cluster = self._config_data.clusters[self._config_data.cluster_name]
