@@ -5,10 +5,9 @@ import shlex
 import signal
 from contextlib import suppress
 from dataclasses import dataclass, field
-from datetime import datetime
+from datetime import datetime, timezone
 from typing import Any, AsyncIterator, Dict, Iterable, List, Mapping, Optional, Sequence
 
-import async_timeout
 import attr
 import psutil
 from aiodocker.exceptions import DockerError
@@ -99,6 +98,18 @@ class JobStatusHistory:
     exit_code: Optional[int] = None
 
 
+class JobRestartPolicy(str, enum.Enum):
+    NEVER = "never"
+    ON_FAILURE = "on-failure"
+    ALWAYS = "always"
+
+    def __str__(self) -> str:
+        return self.value
+
+    def __repr__(self) -> str:
+        return repr(self.value)
+
+
 @dataclass(frozen=True)
 class JobDescription:
     id: str
@@ -108,12 +119,15 @@ class JobDescription:
     history: JobStatusHistory
     container: Container
     is_preemptible: bool
+    uri: URL
     name: Optional[str] = None
     tags: Sequence[str] = ()
     description: Optional[str] = None
     http_url: URL = URL()
     ssh_server: URL = URL()
     internal_hostname: Optional[str] = None
+    restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER
+    life_span: Optional[float] = None
 
 
 @dataclass(frozen=True)
@@ -140,6 +154,7 @@ class Jobs(metaclass=NoPublicConstructor):
         description: Optional[str] = None,
         is_preemptible: bool = False,
         schedule_timeout: Optional[float] = None,
+        restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER,
         life_span: Optional[float] = None,
     ) -> JobDescription:
         url = self._config.api_url / "jobs"
@@ -155,6 +170,8 @@ class Jobs(metaclass=NoPublicConstructor):
             payload["description"] = description
         if schedule_timeout:
             payload["schedule_timeout"] = schedule_timeout
+        if restart_policy != JobRestartPolicy.NEVER:
+            payload["restart_policy"] = str(restart_policy)
         if life_span is not None:
             payload["max_run_time_minutes"] = int(life_span // 60)
         payload["cluster_name"] = self._config.cluster_name
@@ -170,8 +187,13 @@ class Jobs(metaclass=NoPublicConstructor):
         name: str = "",
         tags: Iterable[str] = (),
         owners: Iterable[str] = (),
-    ) -> List[JobDescription]:
+        since: Optional[datetime] = None,
+        until: Optional[datetime] = None,
+        reverse: bool = False,
+        limit: Optional[int] = None,
+    ) -> AsyncIterator[JobDescription]:
         url = self._config.api_url / "jobs"
+        headers = {"Accept": "application/x-ndjson"}
         params: MultiDict[str] = MultiDict()
         for status in statuses:
             params.add("status", status.value)
@@ -181,11 +203,32 @@ class Jobs(metaclass=NoPublicConstructor):
             params.add("owner", owner)
         for tag in tags:
             params.add("tag", tag)
+        if since:
+            if since.tzinfo is None:
+                # Interpret naive datetime object as local time.
+                since = since.astimezone(timezone.utc)
+            params.add("since", since.isoformat())
+        if until:
+            if until.tzinfo is None:
+                until = until.astimezone(timezone.utc)
+            params.add("until", until.isoformat())
         params["cluster_name"] = self._config.cluster_name
+        if reverse:
+            params.add("reverse", "1")
+        if limit is not None:
+            params.add("limit", str(limit))
         auth = await self._config._api_auth()
-        async with self._core.request("GET", url, params=params, auth=auth) as resp:
-            ret = await resp.json()
-            return [_job_description_from_api(j, self._parse) for j in ret["jobs"]]
+        async with self._core.request(
+            "GET", url, headers=headers, params=params, auth=auth
+        ) as resp:
+            if resp.headers.get("Content-Type", "").startswith("application/x-ndjson"):
+                async for line in resp.content:
+                    j = json.loads(line)
+                    yield _job_description_from_api(j, self._parse)
+            else:
+                ret = await resp.json()
+                for j in ret["jobs"]:
+                    yield _job_description_from_api(j, self._parse)
 
     async def kill(self, id: str) -> None:
         url = self._config.api_url / "jobs" / id
@@ -286,7 +329,6 @@ class Jobs(metaclass=NoPublicConstructor):
         *,
         tty: bool = False,
         no_key_check: bool = False,
-        timeout: Optional[float] = None,
     ) -> int:
         try:
             job_status = await self.status(id)
@@ -318,8 +360,7 @@ class Jobs(metaclass=NoPublicConstructor):
         command += ["-p", str(port), f"{server_url.user}@{server_url.host}", payload]
         proc = await asyncio.create_subprocess_exec(*command)
         try:
-            async with async_timeout.timeout(timeout):
-                return await proc.wait()
+            return await proc.wait()
         finally:
             await _kill_proc_tree(proc.pid, timeout=10)
             # add a sleep to get process watcher a chance to execute all callbacks
@@ -489,7 +530,7 @@ def _container_from_api(data: Dict[str, Any], parse: Parser) -> Container:
     try:
         image = parse.remote_image(data["image"])
     except ValueError:
-        image = RemoteImage(name=INVALID_IMAGE_NAME)
+        image = RemoteImage.new_external_image(name=INVALID_IMAGE_NAME)
 
     return Container(
         image=image,
@@ -499,6 +540,7 @@ def _container_from_api(data: Dict[str, Any], parse: Parser) -> Container:
         http=_http_port_from_api(data["http"]) if "http" in data else None,
         env=data.get("env", dict()),
         volumes=[_volume_from_api(v) for v in data.get("volumes", [])],
+        tty=data.get("tty", False),
     )
 
 
@@ -542,6 +584,11 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
     http_url_named = URL(res.get("http_url_named", ""))
     ssh_server = URL(res.get("ssh_server", ""))
     internal_hostname = res.get("internal_hostname", None)
+    restart_policy = JobRestartPolicy(res.get("restart_policy", JobRestartPolicy.NEVER))
+    max_run_time_minutes = res.get("max_run_time_minutes")
+    life_span = (
+        max_run_time_minutes * 60.0 if max_run_time_minutes is not None else None
+    )
     return JobDescription(
         status=JobStatus(res["status"]),
         id=res["id"],
@@ -556,6 +603,9 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
         http_url=http_url_named or http_url,
         ssh_server=ssh_server,
         internal_hostname=internal_hostname,
+        uri=URL(res["uri"]),
+        restart_policy=restart_policy,
+        life_span=life_span,
     )
 
 
