@@ -3,6 +3,8 @@ import datetime
 import enum
 import errno
 import fnmatch
+import json
+import logging
 import os
 import re
 import time
@@ -17,9 +19,9 @@ from typing import (
     Callable,
     Dict,
     Iterable,
-    List,
     Optional,
     Tuple,
+    cast,
 )
 
 import aiohttp
@@ -47,12 +49,13 @@ from .users import Action
 from .utils import NoPublicConstructor, retries
 
 
+log = logging.getLogger(__name__)
+
 MAX_OPEN_FILES = 20
 READ_SIZE = 2 ** 20  # 1 MiB
 TIME_THRESHOLD = 1.0
 
 Printer = Callable[[str], None]
-ProgressQueueItem = Optional[Tuple[Callable[[Any], None], Any]]
 
 
 class FileStatusType(str, enum.Enum):
@@ -126,19 +129,23 @@ class Storage(metaclass=NoPublicConstructor):
             < self._max_time_diff + TIME_THRESHOLD + 1.0
         )
 
-    async def ls(self, uri: URL) -> List[FileStatus]:
+    async def ls(self, uri: URL) -> AsyncIterator[FileStatus]:
         url = self._config.storage_url / self._uri_to_path(uri)
         url = url.with_query(op="LISTSTATUS")
+        headers = {"Accept": "application/x-ndjson"}
 
         request_time = time.time()
         auth = await self._config._api_auth()
-        async with self._core.request("GET", url, auth=auth) as resp:
+        async with self._core.request("GET", url, headers=headers, auth=auth) as resp:
             self._set_time_diff(request_time, resp)
-            res = await resp.json()
-            return [
-                _file_status_from_api(status)
-                for status in res["FileStatuses"]["FileStatus"]
-            ]
+            if resp.headers.get("Content-Type", "").startswith("application/x-ndjson"):
+                async for line in resp.content:
+                    status = json.loads(line)["FileStatus"]
+                    yield _file_status_from_api(status)
+            else:
+                res = await resp.json()
+                for status in res["FileStatuses"]["FileStatus"]:
+                    yield _file_status_from_api(status)
 
     async def glob(self, uri: URL, *, dironly: bool = False) -> AsyncIterator[URL]:
         if not _has_magic(uri.path):
@@ -185,7 +192,7 @@ class Storage(metaclass=NoPublicConstructor):
         yield uri
 
     async def _iterdir(self, uri: URL, dironly: bool) -> AsyncIterator[FileStatus]:
-        for stat in await self.ls(uri):
+        async for stat in self.ls(uri):
             if not dironly or stat.is_dir():
                 yield stat
 
@@ -302,33 +309,22 @@ class Storage(metaclass=NoPublicConstructor):
     # high-level helpers
 
     async def _iterate_file(
-        self,
-        src: Path,
-        dst: URL,
-        *,
-        progress: AbstractFileProgress,
-        queue: "asyncio.Queue[ProgressQueueItem]",
+        self, src: Path, dst: URL, *, progress: "QueuedProgress",
     ) -> AsyncIterator[bytes]:
         loop = asyncio.get_event_loop()
         src_url = URL(src.as_uri())
         async with self._file_sem:
             with src.open("rb") as stream:
                 size = os.stat(stream.fileno()).st_size
-                await queue.put(
-                    (progress.start, StorageProgressStart(src_url, dst, size))
-                )
+                await progress.start(StorageProgressStart(src_url, dst, size))
                 chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
                 pos = len(chunk)
                 while chunk:
-                    await queue.put(
-                        (progress.step, StorageProgressStep(src_url, dst, pos, size))
-                    )
+                    await progress.step(StorageProgressStep(src_url, dst, pos, size))
                     yield chunk
                     chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
                     pos += len(chunk)
-                await queue.put(
-                    (progress.complete, StorageProgressComplete(src_url, dst, size))
-                )
+                await progress.complete(StorageProgressComplete(src_url, dst, size))
 
     async def upload_file(
         self,
@@ -338,8 +334,6 @@ class Storage(metaclass=NoPublicConstructor):
         update: bool = False,
         progress: Optional[AbstractFileProgress] = None,
     ) -> None:
-        if progress is None:
-            progress = _DummyProgress()
         src = normalize_local_path_uri(src)
         dst = normalize_storage_path_uri(
             dst, self._config.username, self._config.cluster_name
@@ -385,24 +379,17 @@ class Storage(metaclass=NoPublicConstructor):
                         src_stat, dst_stat
                     ):
                         return
-        queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
-        await _run_progress(
-            queue, self._upload_file(path, dst, progress=progress, queue=queue)
-        )
+
+        queued = QueuedProgress(progress)
+        await run_progress(queued, self._upload_file(path, dst, progress=queued))
 
     async def _upload_file(
-        self,
-        src_path: Path,
-        dst: URL,
-        *,
-        progress: AbstractFileProgress,
-        queue: "asyncio.Queue[ProgressQueueItem]",
+        self, src_path: Path, dst: URL, *, progress: "QueuedProgress",
     ) -> None:
         for retry in retries(f"Fail to upload {dst}"):
             async with retry:
                 await self.create(
-                    dst,
-                    self._iterate_file(src_path, dst, progress=progress, queue=queue),
+                    dst, self._iterate_file(src_path, dst, progress=progress),
                 )
 
     async def upload_dir(
@@ -414,8 +401,6 @@ class Storage(metaclass=NoPublicConstructor):
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
-        if progress is None:
-            progress = _DummyProgress()
         if filter is None:
             filter = _always
         src = normalize_local_path_uri(src)
@@ -427,18 +412,11 @@ class Storage(metaclass=NoPublicConstructor):
             raise FileNotFoundError(errno.ENOENT, "No such file", str(path))
         if not path.is_dir():
             raise NotADirectoryError(errno.ENOTDIR, "Not a directory", str(path))
-        queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
-        await _run_progress(
-            queue,
+        queued = QueuedProgress(progress)
+        await run_progress(
+            queued,
             self._upload_dir(
-                src,
-                path,
-                dst,
-                "",
-                update=update,
-                filter=filter,
-                progress=progress,
-                queue=queue,
+                src, path, dst, "", update=update, filter=filter, progress=queued,
             ),
         )
 
@@ -451,8 +429,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         filter: Callable[[str], Awaitable[bool]],
-        progress: AbstractRecursiveFileProgress,
-        queue: "asyncio.Queue[ProgressQueueItem]",
+        progress: "QueuedProgress",
     ) -> None:
         tasks = []
         try:
@@ -463,7 +440,7 @@ class Storage(metaclass=NoPublicConstructor):
                         async with retry:
                             dst_files = {
                                 item.name: item
-                                for item in await self.ls(dst)
+                                async for item in self.ls(dst)
                                 if item.is_file()
                             }
                     exists = True
@@ -475,7 +452,7 @@ class Storage(metaclass=NoPublicConstructor):
                         await self.mkdir(dst, exist_ok=True)
         except FileExistsError:
             raise NotADirectoryError(errno.ENOTDIR, "Not a directory", str(dst))
-        await queue.put((progress.enter, StorageProgressEnterDir(src, dst)))
+        await progress.enter(StorageProgressEnterDir(src, dst))
         loop = asyncio.get_event_loop()
         async with self._file_sem:
             folder = await loop.run_in_executor(None, lambda: list(src_path.iterdir()))
@@ -483,6 +460,7 @@ class Storage(metaclass=NoPublicConstructor):
             name = child.name
             child_rel_path = f"{rel_path}/{name}" if rel_path else name
             if not await filter(child_rel_path):
+                log.debug(f"Skip {child_rel_path}")
                 continue
             if child.is_file():
                 if (
@@ -492,9 +470,7 @@ class Storage(metaclass=NoPublicConstructor):
                 ):
                     continue
                 tasks.append(
-                    self._upload_file(
-                        src_path / name, dst / name, progress=progress, queue=queue
-                    )
+                    self._upload_file(src_path / name, dst / name, progress=progress)
                 )
             elif child.is_dir():
                 tasks.append(
@@ -506,25 +482,21 @@ class Storage(metaclass=NoPublicConstructor):
                         update=update,
                         filter=filter,
                         progress=progress,
-                        queue=queue,
                     )
                 )
             else:
                 # This case is for uploading non-regular file,
                 # e.g. blocking device or unix socket
                 # Coverage temporary skipped, the line is waiting for a champion
-                await queue.put(
-                    (
-                        progress.fail,
-                        StorageProgressFail(
-                            src / name,
-                            dst / name,
-                            f"Cannot upload {child}, not regular file/directory",
-                        ),
-                    )
+                await progress.fail(
+                    StorageProgressFail(
+                        src / name,
+                        dst / name,
+                        f"Cannot upload {child}, not regular file/directory",
+                    ),
                 )  # pragma: no cover
-        await _run_concurrently(tasks)
-        await queue.put((progress.leave, StorageProgressLeaveDir(src, dst)))
+        await run_concurrently(tasks)
+        await progress.leave(StorageProgressLeaveDir(src, dst))
 
     async def download_file(
         self,
@@ -534,8 +506,6 @@ class Storage(metaclass=NoPublicConstructor):
         update: bool = False,
         progress: Optional[AbstractFileProgress] = None,
     ) -> None:
-        if progress is None:
-            progress = _DummyProgress()
         src = normalize_storage_path_uri(
             src, self._config.username, self._config.cluster_name
         )
@@ -554,12 +524,9 @@ class Storage(metaclass=NoPublicConstructor):
                     dst_stat, src_stat
                 ):
                     return
-        queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
-        await _run_progress(
-            queue,
-            self._download_file(
-                src, dst, path, src_stat.size, progress=progress, queue=queue
-            ),
+        queued = QueuedProgress(progress)
+        await run_progress(
+            queued, self._download_file(src, dst, path, src_stat.size, progress=queued),
         )
 
     async def _download_file(
@@ -569,28 +536,22 @@ class Storage(metaclass=NoPublicConstructor):
         dst_path: Path,
         size: int,
         *,
-        progress: AbstractFileProgress,
-        queue: "asyncio.Queue[ProgressQueueItem]",
+        progress: "QueuedProgress",
     ) -> None:
         loop = asyncio.get_event_loop()
         async with self._file_sem:
-            with dst_path.open("wb") as stream:
-                await queue.put((progress.start, StorageProgressStart(src, dst, size)))
-                for retry in retries(f"Fail to download {src}"):
-                    async with retry:
+            await progress.start(StorageProgressStart(src, dst, size))
+            for retry in retries(f"Fail to download {src}"):
+                async with retry:
+                    with dst_path.open("wb") as stream:
                         pos = 0
                         async for chunk in self.open(src):
                             pos += len(chunk)
-                            await queue.put(
-                                (
-                                    progress.step,
-                                    StorageProgressStep(src, dst, pos, size),
-                                )
+                            await progress.step(
+                                StorageProgressStep(src, dst, pos, size)
                             )
                             await loop.run_in_executor(None, stream.write, chunk)
-                await queue.put(
-                    (progress.complete, StorageProgressComplete(src, dst, size))
-                )
+            await progress.complete(StorageProgressComplete(src, dst, size))
 
     async def download_dir(
         self,
@@ -601,8 +562,6 @@ class Storage(metaclass=NoPublicConstructor):
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
-        if progress is None:
-            progress = _DummyProgress()
         if filter is None:
             filter = _always
         src = normalize_storage_path_uri(
@@ -610,18 +569,11 @@ class Storage(metaclass=NoPublicConstructor):
         )
         dst = normalize_local_path_uri(dst)
         path = _extract_path(dst)
-        queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
-        await _run_progress(
-            queue,
+        queued = QueuedProgress(progress)
+        await run_progress(
+            queued,
             self._download_dir(
-                src,
-                dst,
-                path,
-                "",
-                update=update,
-                filter=filter,
-                progress=progress,
-                queue=queue,
+                src, dst, path, "", update=update, filter=filter, progress=queued
             ),
         )
 
@@ -634,11 +586,10 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         filter: Callable[[str], Awaitable[bool]],
-        progress: AbstractRecursiveFileProgress,
-        queue: "asyncio.Queue[ProgressQueueItem]",
+        progress: "QueuedProgress",
     ) -> None:
         dst_path.mkdir(parents=True, exist_ok=True)
-        await queue.put((progress.enter, StorageProgressEnterDir(src, dst)))
+        await progress.enter(StorageProgressEnterDir(src, dst))
         tasks = []
         if update:
             loop = asyncio.get_event_loop()
@@ -652,12 +603,13 @@ class Storage(metaclass=NoPublicConstructor):
 
         for retry in retries(f"Fail to list {src}"):
             async with retry:
-                folder = await self.ls(src)
+                folder = [item async for item in self.ls(src)]
 
         for child in folder:
             name = child.name
             child_rel_path = f"{rel_path}/{name}" if rel_path else name
             if not await filter(child_rel_path):
+                log.debug(f"Skip {child_rel_path}")
                 continue
             if child.is_file():
                 if (
@@ -673,7 +625,6 @@ class Storage(metaclass=NoPublicConstructor):
                         dst_path / name,
                         child.size,
                         progress=progress,
-                        queue=queue,
                     )
                 )
             elif child.is_dir():
@@ -686,22 +637,18 @@ class Storage(metaclass=NoPublicConstructor):
                         update=update,
                         filter=filter,
                         progress=progress,
-                        queue=queue,
                     )
                 )
             else:
-                await queue.put(
-                    (
-                        progress.fail,
-                        StorageProgressFail(
-                            src / name,
-                            dst / name,
-                            f"Cannot download {child}, not regular file/directory",
-                        ),
-                    )
+                await progress.fail(
+                    StorageProgressFail(
+                        src / name,
+                        dst / name,
+                        f"Cannot download {child}, not regular file/directory",
+                    ),
                 )  # pragma: no cover
-        await _run_concurrently(tasks)
-        await queue.put((progress.leave, StorageProgressLeaveDir(src, dst)))
+        await run_concurrently(tasks)
+        await progress.leave(StorageProgressLeaveDir(src, dst))
 
 
 _magic_check = re.compile("(?:[*?[])")
@@ -729,27 +676,56 @@ def _file_status_from_api(values: Dict[str, Any]) -> FileStatus:
     )
 
 
-async def _run_concurrently(coros: Iterable[Awaitable[Any]]) -> None:
-    loop = asyncio.get_event_loop()
-    tasks: "Iterable[asyncio.Future[Any]]" = [loop.create_task(coro) for coro in coros]
-    if not tasks:
-        return
-    try:
-        done, tasks = await asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
-        for task in done:
-            await task
-    except:  # noqa: E722
-        for task in tasks:
-            task.cancel()
-        # wait for actual cancellation, ignore all exceptions raised from tasks
-        if tasks:
-            await asyncio.wait(tasks)
-        raise  # pragma: no cover
+ProgressQueueItem = Optional[Tuple[Callable[[Any], None], Any]]
 
 
-async def _run_progress(
-    queue: "asyncio.Queue[ProgressQueueItem]", coro: Awaitable[None]
+class QueuedProgress:
+    def __init__(
+        self, progress: Optional[AbstractFileProgress],
+    ):
+        self._progress = progress
+        self._queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
+
+    @property
+    def queue(self) -> "asyncio.Queue[ProgressQueueItem]":
+        return self._queue
+
+    async def start(self, data: StorageProgressStart) -> None:
+        if self._progress is not None:
+            await self._queue.put((self._progress.start, data))
+
+    async def complete(self, data: StorageProgressComplete) -> None:
+        if self._progress is not None:
+            await self._queue.put((self._progress.complete, data))
+
+    async def step(self, data: StorageProgressStep) -> None:
+        if self._progress is not None:
+            await self._queue.put((self._progress.step, data))
+
+    async def enter(self, data: StorageProgressEnterDir) -> None:
+        if self._progress is not None:
+            progress = cast(AbstractRecursiveFileProgress, self._progress)
+            await self._queue.put((progress.enter, data))
+
+    async def leave(self, data: StorageProgressLeaveDir) -> None:
+        if self._progress is not None:
+            progress = cast(AbstractRecursiveFileProgress, self._progress)
+            await self._queue.put((progress.leave, data))
+
+    async def fail(self, data: StorageProgressFail) -> None:
+        if self._progress is not None:
+            progress = cast(AbstractRecursiveFileProgress, self._progress)
+            await self._queue.put((progress.fail, data))
+
+
+async def run_progress(
+    progress: Optional[QueuedProgress], coro: Awaitable[None]
 ) -> None:
+    if progress is None:
+        return await coro
+
+    queue = progress.queue
+
     async def wrapped() -> None:
         try:
             await coro
@@ -767,24 +743,22 @@ async def _run_progress(
     await task
 
 
-class _DummyProgress(AbstractRecursiveFileProgress):
-    def start(self, data: StorageProgressStart) -> None:
-        pass
-
-    def complete(self, data: StorageProgressComplete) -> None:
-        pass
-
-    def step(self, data: StorageProgressStep) -> None:
-        pass
-
-    def enter(self, data: StorageProgressEnterDir) -> None:
-        pass
-
-    def leave(self, data: StorageProgressLeaveDir) -> None:
-        pass
-
-    def fail(self, data: StorageProgressFail) -> None:
-        pass
+async def run_concurrently(coros: Iterable[Awaitable[Any]]) -> None:
+    loop = asyncio.get_event_loop()
+    tasks: "Iterable[asyncio.Future[Any]]" = [loop.create_task(coro) for coro in coros]
+    if not tasks:
+        return
+    try:
+        done, tasks = await asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
+        for task in done:
+            await task
+    except:  # noqa: E722
+        for task in tasks:
+            task.cancel()
+        # wait for actual cancellation, ignore all exceptions raised from tasks
+        if tasks:
+            await asyncio.wait(tasks)
+        raise  # pragma: no cover
 
 
 async def _always(path: str) -> bool:
