@@ -9,12 +9,19 @@ from typing import Iterable, Iterator, List
 import humanize
 from click import style, unstyle
 
-from neuromation.api import JobDescription, JobStatus, JobTelemetry, Resources
+from neuromation.api import (
+    JobDescription,
+    JobRestartPolicy,
+    JobStatus,
+    JobTelemetry,
+    Resources,
+)
 from neuromation.cli.parse_utils import JobColumnInfo
 from neuromation.cli.printer import StreamPrinter, TTYPrinter
 from neuromation.cli.utils import format_size
 
 from .ftable import table
+from .utils import ImageFormatter, URIFormatter, image_formatter
 
 
 COLORS = {
@@ -30,6 +37,26 @@ def format_job_status(status: JobStatus) -> str:
     return style(status.value, fg=COLORS.get(status, "reset"))
 
 
+def format_timedelta(delta: datetime.timedelta) -> str:
+    s = int(delta.total_seconds())
+    if s < 0:
+        raise ValueError(f"Invalid delta {delta}: expect non-negative total value")
+    _sec_in_minute = 60
+    _sec_in_hour = _sec_in_minute * 60
+    _sec_in_day = _sec_in_hour * 24
+    d, s = divmod(s, _sec_in_day)
+    h, s = divmod(s, _sec_in_hour)
+    m, s = divmod(s, _sec_in_minute)
+    return "".join(
+        [
+            f"{d}d" if d else "",
+            f"{h}h" if h else "",
+            f"{m}m" if m else "",
+            f"{s}s" if s else "",
+        ]
+    )
+
+
 class JobFormatter:
     def __init__(self, quiet: bool = True) -> None:
         self._quiet = quiet
@@ -74,6 +101,10 @@ class JobFormatter:
 
 
 class JobStatusFormatter:
+    def __init__(self, uri_formatter: URIFormatter) -> None:
+        self._format_uri = uri_formatter
+        self._format_image = image_formatter(uri_formatter=uri_formatter)
+
     def __call__(self, job_status: JobDescription) -> str:
         result: str = f"Job: {job_status.id}\n"
         if job_status.name:
@@ -91,7 +122,7 @@ class JobStatusFormatter:
             and job_status.status in [JobStatus.FAILED, JobStatus.PENDING]
         ):
             result += f" ({job_status.history.reason})"
-        result += f"\nImage: {job_status.container.image}\n"
+        result += f"\nImage: {self._format_image(job_status.container.image)}\n"
 
         if job_status.container.entrypoint:
             result += f"Entrypoint: {job_status.container.entrypoint}\n"
@@ -99,6 +130,15 @@ class JobStatusFormatter:
         resource_formatter = ResourcesFormatter()
         result += resource_formatter(job_status.container.resources) + "\n"
         result += f"Preemptible: {job_status.is_preemptible}\n"
+        if job_status.restart_policy != JobRestartPolicy.NEVER:
+            result += f"Restart policy: {job_status.restart_policy}\n"
+        if job_status.life_span is not None:
+            limit = (
+                "no limit"
+                if job_status.life_span == 0
+                else format_timedelta(datetime.timedelta(seconds=job_status.life_span))
+            )
+            result += f"Life span: {limit}\n"
 
         if job_status.container.tty:
             result += "TTY: True\n"
@@ -107,7 +147,7 @@ class JobStatusFormatter:
             rows = [
                 (
                     volume.container_path,
-                    f"{volume.storage_uri}",
+                    f"{self._format_uri(volume.storage_uri)}",
                     "READONLY" if volume.read_only else " ",
                 )
                 for volume in job_status.container.volumes
@@ -222,7 +262,9 @@ class TabularJobRow:
     command: str
 
     @classmethod
-    def from_job(cls, job: JobDescription, username: str) -> "TabularJobRow":
+    def from_job(
+        cls, job: JobDescription, username: str, image_formatter: ImageFormatter
+    ) -> "TabularJobRow":
         if job.status == JobStatus.PENDING:
             when = job.history.created_at
         elif job.status == JobStatus.RUNNING:
@@ -235,14 +277,14 @@ class TabularJobRow:
         if delta < datetime.timedelta(days=1):
             when_humanized = humanize.naturaltime(delta)
         else:
-            when_humanized = humanize.naturaldate(when)
+            when_humanized = humanize.naturaldate(when.astimezone())
         return cls(
             id=job.id,
             name=job.name if job.name else "",
             tags=",".join(job.tags),
             status=job.status,
             when=when_humanized,
-            image=str(job.container.image),
+            image=image_formatter(job.container.image),
             owner=("<you>" if job.owner == username else job.owner),
             description=job.description if job.description else "",
             cluster_name=job.cluster_name,
@@ -254,17 +296,26 @@ class TabularJobRow:
 
 
 class TabularJobsFormatter(BaseJobsFormatter):
-    def __init__(self, width: int, username: str, columns: List[JobColumnInfo]):
+    def __init__(
+        self,
+        width: int,
+        username: str,
+        columns: List[JobColumnInfo],
+        image_formatter: ImageFormatter,
+    ) -> None:
         self.width = width
         self._username = username
         self._columns = columns
+        self._image_formatter = image_formatter
 
     def __call__(self, jobs: Iterable[JobDescription]) -> Iterator[str]:
         rows: List[List[str]] = []
         rows.append([column.title for column in self._columns])
         for job in jobs:
             rows.append(
-                TabularJobRow.from_job(job, self._username).to_list(self._columns)
+                TabularJobRow.from_job(
+                    job, self._username, image_formatter=self._image_formatter
+                ).to_list(self._columns)
             )
         for line in table(
             rows,
