@@ -8,11 +8,12 @@ import sys
 import textwrap
 import uuid
 import webbrowser
-from datetime import timedelta
+from datetime import datetime, timedelta
 from typing import Dict, Iterator, List, Optional, Sequence, Set, Tuple
 
 import async_timeout
 import click
+from dateutil.parser import isoparse
 from yarl import URL
 
 from neuromation.api import (
@@ -21,13 +22,28 @@ from neuromation.api import (
     Container,
     HTTPPort,
     JobDescription,
+    JobRestartPolicy,
     JobStatus,
     RemoteImage,
     Resources,
     Volume,
 )
 from neuromation.cli.formatters import DockerImageProgress
+from neuromation.cli.formatters.utils import (
+    URIFormatter,
+    image_formatter,
+    uri_formatter,
+)
 
+from .click_types import (
+    JOB,
+    JOB_COLUMNS,
+    JOB_NAME,
+    LOCAL_REMOTE_PORT,
+    MEGABYTE,
+    PRESET,
+    ImageType,
+)
 from .const import EX_PLATFORMERROR
 from .defaults import (
     GPU_MODELS,
@@ -48,14 +64,8 @@ from .formatters.jobs import (
 from .parse_utils import JobColumnInfo, get_default_columns, parse_columns
 from .root import Root
 from .utils import (
-    JOB_COLUMNS,
-    JOB_NAME,
-    LOCAL_REMOTE_PORT,
-    MEGABYTE,
     NEURO_STEAL_CONFIG,
-    PRESET,
     AsyncExitStack,
-    ImageType,
     alias,
     argument,
     command,
@@ -257,6 +267,13 @@ def job() -> None:
     help="File with environment variables to pass",
     secure=True,
 )
+@option(
+    "--restart",
+    default="never",
+    show_default=True,
+    type=click.Choice([str(i) for i in JobRestartPolicy]),
+    help="Restart policy to apply when a job exits",
+)
 @option(
     "--life-span",
     type=str,
@@ -303,6 +320,7 @@ async def submit(
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
+    restart: str,
     life_span: Optional[str],
     preemptible: bool,
     name: Optional[str],
@@ -350,6 +368,7 @@ async def submit(
         volume=volume,
         env=env,
         env_file=env_file,
+        restart=restart,
         life_span=life_span,
         preemptible=preemptible,
         name=name,
@@ -364,7 +383,7 @@ async def submit(
 
 
 @command(context_settings=dict(allow_interspersed_args=False))
-@argument("job")
+@argument("job", type=JOB)
 @argument("cmd", nargs=-1, type=click.UNPROCESSED, required=True)
 @option(
     "-t/-T",
@@ -373,10 +392,21 @@ async def submit(
     is_flag=True,
     help="Allocate virtual tty. Useful for interactive jobs.",
 )
+@option(
+    "-i/-I",
+    "--interactive/--no-interactive",
+    default=None,
+    is_flag=True,
+    help=(
+        "Keep STDIN open even if not attached. "
+        "On for tty by default, false otherwise."
+    ),
+)
 @option(
     "--no-key-check",
     is_flag=True,
     help="Disable host key checks. Should be used with caution.",
+    hidden=True,
 )
 @option(
     "--timeout",
@@ -389,6 +419,7 @@ async def exec(
     root: Root,
     job: str,
     tty: bool,
+    interactive: Optional[bool],
     no_key_check: bool,
     cmd: Sequence[str],
     timeout: float,
@@ -408,18 +439,17 @@ async def exec(
     id = await resolve_job(
         job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
     )
-    retcode = await root.client.jobs.exec(
-        id,
-        shlex.split(real_cmd),
-        tty=tty,
-        no_key_check=no_key_check,
-        timeout=timeout if timeout else None,
-    )
-    sys.exit(retcode)
+    if interactive is None:
+        interactive = tty
+    async with async_timeout.timeout(timeout if timeout else None,):
+        retcode = await root.client.jobs.exec(
+            id, shlex.split(real_cmd), tty=tty, no_key_check=no_key_check,
+        )
+        sys.exit(retcode)
 
 
 @command()
-@argument("job")
+@argument("job", type=JOB)
 @argument("local_remote_port", type=LOCAL_REMOTE_PORT, nargs=-1, required=True)
 @option(
     "--no-key-check",
@@ -472,7 +502,7 @@ async def port_forward(
 
 
 @command()
-@argument("job")
+@argument("job", type=JOB)
 async def logs(root: Root, job: str) -> None:
     """
     Print the logs for a container.
@@ -494,7 +524,7 @@ async def _print_logs(root: Root, job: str) -> None:
     async for chunk in root.client.jobs.monitor(job):
         if not chunk:
             break
-        click.echo(chunk.decode(errors="ignore"), nl=False)
+        click.echo(chunk.decode(errors="replace"), nl=False)
 
 
 @command()
@@ -515,16 +545,6 @@ async def _print_logs(root: Root, job: str) -> None:
     help="Filter out jobs by owner (multiple option).",
     secure=True,
 )
-@option(
-    "-a",
-    "--all",
-    is_flag=True,
-    default=False,
-    help=(
-        "Show all jobs regardless the status (equivalent to "
-        "`-s pending -s running -s succeeded -s failed`)."
-    ),
-)
 @option("-n", "--name", metavar="NAME", help="Filter out jobs by name.", secure=True)
 @option(
     "-t",
@@ -542,6 +562,26 @@ async def _print_logs(root: Root, job: str) -> None:
     help="Filter out jobs by description (exact match).",
     secure=True,
 )
+@option(
+    "--since",
+    metavar="DATE",
+    help="Show jobs created after a specific date (including).",
+)
+@option(
+    "--until",
+    metavar="DATE",
+    help="Show jobs created before a specific date (including).",
+)
+@option(
+    "-a",
+    "--all",
+    is_flag=True,
+    default=False,
+    help=(
+        "Show all jobs regardless the status (equivalent to "
+        "`-s pending -s running -s succeeded -s failed`)."
+    ),
+)
 @deprecated_quiet_option
 @option("-w", "--wide", is_flag=True, help="Do not cut long lines for terminal width.")
 @option(
@@ -555,6 +595,7 @@ async def _print_logs(root: Root, job: str) -> None:
     ),
     default=None,
 )
+@option("--full-uri", is_flag=True, help="Output full image URI.")
 async def ls(
     root: Root,
     status: Sequence[str],
@@ -562,9 +603,12 @@ async def ls(
     name: str,
     tag: Sequence[str],
     owner: Sequence[str],
+    since: str,
+    until: str,
     description: str,
     wide: bool,
     format: Optional[List[JobColumnInfo]],
+    full_uri: bool,
 ) -> None:
     """
     List all jobs.
@@ -584,14 +628,26 @@ async def ls(
     statuses = calc_statuses(status, all)
     owners = set(owner)
     tags = set(tag)
-    jobs = await root.client.jobs.list(
-        statuses=statuses, name=name, owners=owners, tags=tags
+    jobs = root.client.jobs.list(
+        statuses=statuses,
+        name=name,
+        owners=owners,
+        tags=tags,
+        since=_parse_date(since),
+        until=_parse_date(until),
     )
 
     # client-side filtering
     if description:
-        jobs = [job for job in jobs if job.description == description]
+        jobs = (job async for job in jobs if job.description == description)
 
+    uri_fmtr: URIFormatter
+    if full_uri:
+        uri_fmtr = str
+    else:
+        uri_fmtr = uri_formatter(
+            username=root.client.username, cluster_name=root.client.cluster_name
+        )
     if root.quiet:
         formatter: BaseJobsFormatter = SimpleJobsFormatter()
     else:
@@ -599,14 +655,18 @@ async def ls(
             width = 0
         else:
             width = root.terminal_size[0]
-        formatter = TabularJobsFormatter(width, root.client.username, format)
+        image_fmtr = image_formatter(uri_formatter=uri_fmtr)
+        formatter = TabularJobsFormatter(
+            width, root.client.username, format, image_formatter=image_fmtr
+        )
 
-    pager_maybe(formatter(jobs), root.tty, root.terminal_size)
+    pager_maybe(formatter([job async for job in jobs]), root.tty, root.terminal_size)
 
 
 @command()
-@argument("job")
-async def status(root: Root, job: str) -> None:
+@argument("job", type=JOB)
+@option("--full-uri", is_flag=True, help="Output full URI.")
+async def status(root: Root, job: str, full_uri: bool) -> None:
     """
     Display status of a job.
     """
@@ -621,7 +681,14 @@ async def status(root: Root, job: str) -> None:
         },
     )
     res = await root.client.jobs.status(id)
-    click.echo(JobStatusFormatter()(res))
+    uri_fmtr: URIFormatter
+    if full_uri:
+        uri_fmtr = str
+    else:
+        uri_fmtr = uri_formatter(
+            username=root.client.username, cluster_name=root.client.cluster_name
+        )
+    click.echo(JobStatusFormatter(uri_formatter=uri_fmtr)(res))
 
 
 @command()
@@ -634,7 +701,7 @@ async def tags(root: Root) -> None:
 
 
 @command()
-@click.argument("job")
+@click.argument("job", type=JOB)
 async def browse(root: Root, job: str) -> None:
     """
     Opens a job's URL in a web browser.
@@ -647,7 +714,7 @@ async def browse(root: Root, job: str) -> None:
 
 
 @command()
-@argument("job")
+@argument("job", type=JOB)
 @option(
     "--timeout",
     default=0,
@@ -674,7 +741,7 @@ async def top(root: Root, job: str, timeout: float) -> None:
 
 
 @command()
-@argument("job")
+@argument("job", type=JOB)
 @argument("image", type=ImageType())
 async def save(root: Root, job: str, image: RemoteImage) -> None:
     """
@@ -702,7 +769,7 @@ async def save(root: Root, job: str, image: RemoteImage) -> None:
 
 
 @command()
-@argument("jobs", nargs=-1, required=True)
+@argument("jobs", nargs=-1, required=True, type=JOB)
 async def kill(root: Root, jobs: Sequence[str]) -> None:
     """
     Kill job(s).
@@ -836,6 +903,13 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     help="File with environment variables to pass",
     secure=True,
 )
+@option(
+    "--restart",
+    default="never",
+    show_default=True,
+    type=click.Choice([str(i) for i in JobRestartPolicy]),
+    help="Restart policy to apply when a job exits",
+)
 @option(
     "--life-span",
     type=str,
@@ -878,6 +952,7 @@ async def run(
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
+    restart: str,
     life_span: Optional[str],
     preemptible: Optional[bool],
     name: Optional[str],
@@ -933,6 +1008,7 @@ async def run(
         volume=volume,
         env=env,
         env_file=env_file,
+        restart=restart,
         life_span=life_span,
         preemptible=job_preset.is_preemptible,
         name=name,
@@ -982,6 +1058,7 @@ async def run_job(
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
+    restart: str,
     life_span: Optional[str],
     preemptible: bool,
     name: Optional[str],
@@ -1007,6 +1084,9 @@ async def run_job(
     if not wait_start:
         detach = True
 
+    job_restart_policy = JobRestartPolicy(restart)
+    log.debug(f"Job restart policy: {job_restart_policy}")
+
     job_life_span = await calc_life_span(root.client, life_span)
     log.debug(f"Job run-time limit: {job_life_span}")
 
@@ -1065,6 +1145,7 @@ async def run_job(
         name=name,
         tags=tags,
         description=description,
+        restart_policy=job_restart_policy,
         life_span=job_life_span,
     )
     click.echo(JobFormatter(root.quiet)(job))
@@ -1077,7 +1158,11 @@ async def run_job(
     if browse and job.status != JobStatus.FAILED:
         await browse_job(root, job)
 
-    exit_code = None
+    # Even if we detached, but the job has failed to start
+    # (most common reason - no resources), the command fails
+    if job.status == JobStatus.FAILED:
+        sys.exit(job.history.exit_code or EX_PLATFORMERROR)
+
     if not detach:
         if not root.quiet:
             msg = textwrap.dedent(
@@ -1090,15 +1175,12 @@ async def run_job(
             click.echo(click.style(msg, dim=True))
         await _print_logs(root, job.id)
         job = await root.client.jobs.status(job.id)
-        exit_code = job.history.exit_code
-    else:
-        # Even if we detached, but the job has failed to start
-        # (most common reason - no resources), the command fails
+        while job.status in (JobStatus.PENDING, JobStatus.RUNNING):
+            await asyncio.sleep(0.1)
+            job = await root.client.jobs.status(job.id)
         if job.status == JobStatus.FAILED:
-            exit_code = EX_PLATFORMERROR
-
-    if exit_code is not None:
-        sys.exit(exit_code)
+            sys.exit(job.history.exit_code or EX_PLATFORMERROR)
+        sys.exit(job.history.exit_code)
 
     return job
 
@@ -1299,3 +1381,13 @@ def _parse_timedelta(value: str) -> timedelta:
         minutes=int(match.group("m") or 0),
         seconds=int(match.group("s") or 0),
     )
+
+
+def _parse_date(value: str) -> Optional[datetime]:
+    if value:
+        try:
+            return isoparse(value)
+        except ValueError:
+            raise ValueError("Date should be in ISO-8601 format")
+    else:
+        return None
