@@ -1,5 +1,7 @@
 import asyncio
+import io
 import logging
+import os
 import shutil
 import sys
 import warnings
@@ -16,7 +18,17 @@ import neuromation
 from neuromation.api import CONFIG_ENV_NAME, DEFAULT_CONFIG_PATH, ConfigError
 from neuromation.cli.root import Root
 
-from . import admin, completion, config, image, job, project, share, storage
+from . import (
+    admin,
+    blob_storage,
+    completion,
+    config,
+    image,
+    job,
+    project,
+    share,
+    storage,
+)
 from .alias import find_alias
 from .const import (
     EX_DATAERR,
@@ -45,6 +57,29 @@ from .utils import (
 )
 
 
+def setup_stdout(errors: str) -> None:
+    if not isinstance(sys.stdout, io.TextIOWrapper):
+        return
+    sys.stdout.flush()
+    if sys.version_info < (3, 7):
+        buffered = hasattr(sys.stdout.buffer, "raw")
+        buf = open(os.dup(sys.stdout.fileno()), "wb", -1 if buffered else 0)
+        raw = getattr(buf, "raw", buf)
+        raw.name = "<stdout>"
+        sys.stdout = io.TextIOWrapper(
+            buf,
+            encoding=sys.stdout.encoding,
+            errors=errors,
+            line_buffering=sys.stdout.line_buffering,
+            write_through=not buffered,
+        )
+    else:
+        # cast() is a workaround for https://github.com/python/typeshed/issues/3049
+        cast(Any, sys.stdout).reconfigure(errors=errors)
+
+
+setup_stdout(errors="replace")
+
 if sys.platform == "win32":
     if sys.version_info < (3, 7):
         # Python 3.6 has no WindowsProactorEventLoopPolicy class
@@ -454,6 +489,7 @@ cli.add_command(image.image)
 cli.add_command(config.config)
 cli.add_command(completion.completion)
 cli.add_command(share.acl)
+cli.add_command(blob_storage.blob_storage)
 
 cli.add_command(DeprecatedGroup(storage.storage, name="store", hidden=True))
 
