@@ -29,24 +29,12 @@ from typing import (
 
 import click
 import humanize
-from click import BadParameter
 from click.types import convert_type
 from yarl import URL
 
-from neuromation.api import (
-    Action,
-    Client,
-    Factory,
-    JobDescription,
-    JobStatus,
-    LocalImage,
-    RemoteImage,
-    TagOption,
-    Volume,
-)
-from neuromation.api.url_utils import _normalize_uri, uri_from_cli
+from neuromation.api import Action, Client, Factory, JobStatus, TagOption, Volume
+from neuromation.api.url_utils import uri_from_cli
 
-from .parse_utils import JobColumnInfo, parse_columns, to_megabytes
 from .root import Root
 from .stats import upload_gmp_stats
 from .version_utils import run_version_checker
@@ -59,12 +47,6 @@ _T = TypeVar("_T")
 DEPRECATED_HELP_NOTICE = " " + click.style("(DEPRECATED)", fg="red")
 DEPRECATED_INVOKE_NOTICE = "DeprecationWarning: The command {name} is deprecated."
 
-# NOTE: these job name defaults are taken from `platform_api` file `validators.py`
-JOB_NAME_MIN_LENGTH = 3
-JOB_NAME_MAX_LENGTH = 40
-JOB_NAME_PATTERN = "^[a-z](?:-?[a-z0-9])*$"
-JOB_NAME_REGEX = re.compile(JOB_NAME_PATTERN)
-
 NEURO_STEAL_CONFIG = "NEURO_STEAL_CONFIG"
 
 
@@ -115,6 +97,7 @@ def _wrap_async_callback(
     callback: Callable[..., Awaitable[_T]], init_client: bool = True,
 ) -> Callable[..., _T]:
     assert inspect.iscoroutinefunction(callback)
+
     # N.B. the decorator implies @click.pass_obj
     @click.pass_obj
     @functools.wraps(callback)
@@ -401,7 +384,7 @@ def option(*param_decls: Any, **attrs: Any) -> Callable[..., Any]:
     option_attrs = attrs.copy()
     option_attrs.setdefault("cls", Option)
     typ = convert_type(attrs.get("type"), attrs.get("default"))
-    autocompletion = getattr(typ, "autocompletion", None)
+    autocompletion = getattr(typ, "complete", None)
     option_attrs.setdefault("autocompletion", autocompletion)
     return click.option(*param_decls, **option_attrs)
 
@@ -409,7 +392,7 @@ def option(*param_decls: Any, **attrs: Any) -> Callable[..., Any]:
 def argument(*param_decls: Any, **attrs: Any) -> Callable[..., Any]:
     arg_attrs = attrs.copy()
     typ = convert_type(attrs.get("type"), attrs.get("default"))
-    autocompletion = getattr(typ, "autocompletion", None)
+    autocompletion = getattr(typ, "complete", None)
     arg_attrs.setdefault("autocompletion", autocompletion)
     return click.argument(*param_decls, **arg_attrs)
 
@@ -430,8 +413,11 @@ async def resolve_job(
     default_user = client.username
     default_cluster = client.cluster_name
     if id_or_name_or_uri.startswith("job:"):
-        uri = _normalize_uri(
-            id_or_name_or_uri, username=default_user, cluster_name=default_cluster,
+        uri = uri_from_cli(
+            id_or_name_or_uri,
+            username=default_user,
+            cluster_name=default_cluster,
+            allowed_schemes=("job",),
         )
         if uri.host != default_cluster:
             raise ValueError(f"Invalid job URI: cluster_name != '{default_cluster}'")
@@ -450,10 +436,13 @@ async def resolve_job(
     if re.fullmatch(JOB_ID_PATTERN, id_or_name):
         return id_or_name
 
-    jobs: List[JobDescription] = []
     details = f"name={id_or_name}, owner={owner}"
     try:
-        jobs = await client.jobs.list(name=id_or_name, owners={owner})
+        async for job in client.jobs.list(
+            name=id_or_name, owners={owner}, reverse=True, limit=1
+        ):
+            log.debug(f"Job name '{id_or_name}' resolved to job ID '{job.id}'")
+            return job.id
     except asyncio.CancelledError:
         raise
     except Exception as e:
@@ -461,16 +450,11 @@ async def resolve_job(
             f"Failed to resolve job-name {id_or_name_or_uri} resolved as "
             f"{details} to a job-ID: {e}"
         )
-    if jobs:
-        job_id = jobs[-1].id
-        log.debug(f"Job name '{id_or_name}' resolved to job ID '{job_id}'")
-    else:
-        job_id = id_or_name
 
-    return job_id
+    return id_or_name
 
 
-SHARE_SCHEMES = ("storage", "image", "job")
+SHARE_SCHEMES = ("storage", "image", "job", "blob", "role")
 
 
 def parse_resource_for_sharing(uri: str, root: Root) -> URL:
@@ -481,12 +465,16 @@ def parse_resource_for_sharing(uri: str, root: Root) -> URL:
         image = root.client.parse.remote_image(uri, tag_option=TagOption.DENY)
         uri = str(image)
 
-    return uri_from_cli(
+    uri_res = uri_from_cli(
         uri,
         root.client.username,
         root.client.cluster_name,
-        allowed_schemes=("storage", "image", "job"),
+        allowed_schemes=SHARE_SCHEMES,
     )
+    # URI's for object storage can only operate on bucket level
+    if uri_res.scheme == "blob" and "/" in uri_res.path.strip("/"):
+        raise ValueError("Only bucket level permissions are supported for Blob Storage")
+    return uri_res
 
 
 def parse_file_resource(uri: str, root: Root) -> URL:
@@ -501,6 +489,23 @@ def parse_file_resource(uri: str, root: Root) -> URL:
     )
 
 
+def parse_blob_resource(uri: str, root: Root) -> URL:
+    # Username will not be used, just part of the signature
+    return uri_from_cli(
+        uri, root.client.username, root.client.cluster_name, allowed_schemes=("blob",)
+    )
+
+
+def parse_blob_or_file_resource(uri: str, root: Root) -> URL:
+    # Username will not be used, just part of the signature
+    return uri_from_cli(
+        uri,
+        root.client.username,
+        root.client.cluster_name,
+        allowed_schemes=("blob", "file"),
+    )
+
+
 def parse_permission_action(action: str) -> Action:
     try:
         return Action[action.upper()]
@@ -511,162 +516,6 @@ def parse_permission_action(action: str) -> Action:
         )
 
 
-class LocalImageType(click.ParamType):
-    name = "local_image"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> LocalImage:
-        assert ctx is not None
-        root = cast(Root, ctx.obj)
-        client = root.run(root.init_client())
-        return client.parse.local_image(value)
-
-
-class ImageType(click.ParamType):
-    name = "image"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> RemoteImage:
-        assert ctx is not None
-        root = cast(Root, ctx.obj)
-        client = root.run(root.init_client())
-        return client.parse.remote_image(value)
-
-
-class RemoteTaglessImageType(click.ParamType):
-    name = "image"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> RemoteImage:
-        assert ctx is not None
-        root = cast(Root, ctx.obj)
-        client = root.run(root.init_client())
-        return client.parse.remote_image(value, tag_option=TagOption.DENY)
-
-
-class LocalRemotePortParamType(click.ParamType):
-    name = "local-remote-port-pair"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> Tuple[int, int]:
-        try:
-            local_str, remote_str = value.split(":")
-            local, remote = int(local_str), int(remote_str)
-            if not (0 < local <= 65535 and 0 < remote <= 65535):
-                raise ValueError("Port should be in range 1 to 65535")
-            return local, remote
-        except ValueError as e:
-            raise BadParameter(f"{value} is not a valid port combination: {e}")
-
-
-LOCAL_REMOTE_PORT = LocalRemotePortParamType()
-
-
-class MegabyteType(click.ParamType):
-    name = "megabyte"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> int:
-        return to_megabytes(value)
-
-
-MEGABYTE = MegabyteType()
-
-
-class JobNameType(click.ParamType):
-    name = "job_name"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> str:
-        if (
-            len(value) < JOB_NAME_MIN_LENGTH
-            or len(value) > JOB_NAME_MAX_LENGTH
-            or JOB_NAME_REGEX.match(value) is None
-        ):
-            raise ValueError(
-                f"Invalid job name '{value}'.\n"
-                "The name can only contain lowercase letters, numbers and hyphens "
-                "with the following rules: \n"
-                "  - the first character must be a letter; \n"
-                "  - each hyphen must be surrounded by non-hyphen characters; \n"
-                f"  - total length must be between {JOB_NAME_MIN_LENGTH} and "
-                f"{JOB_NAME_MAX_LENGTH} characters long."
-            )
-        return value
-
-
-JOB_NAME = JobNameType()
-
-
-class JobColumnsType(click.ParamType):
-    name = "columns"
-
-    def convert(
-        self,
-        value: Union[str, List[JobColumnInfo]],
-        param: Optional[click.Parameter],
-        ctx: Optional[click.Context],
-    ) -> List[JobColumnInfo]:
-        if isinstance(value, list):
-            return value
-        return parse_columns(value)
-
-
-JOB_COLUMNS = JobColumnsType()
-
-
-class PresetType(click.ParamType):
-    name = "preset"
-
-    def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
-    ) -> str:
-        assert ctx is not None
-        root = cast(Root, ctx.obj)
-        return root.run(self._convert(root, value, param, ctx))
-
-    async def _convert(
-        self,
-        root: Root,
-        value: str,
-        param: Optional[click.Parameter],
-        ctx: Optional[click.Context],
-    ) -> str:
-        client = await root.init_client()
-        if value not in client.presets:
-            raise click.BadParameter(
-                f"Preset {value} is not valid, "
-                "run 'neuro config show' to get a list of available presets",
-                ctx,
-                param,
-            )
-        return value
-
-    def autocompletion(
-        self, ctx: click.Context, args: Sequence[str], incomplete: str
-    ) -> List[Tuple[str, Optional[str]]]:
-        root = cast(Root, ctx.obj)
-        return root.run(self._autocompletion(root, ctx, args, incomplete))
-
-    async def _autocompletion(
-        self, root: Root, ctx: click.Context, args: Sequence[str], incomplete: str
-    ) -> List[Tuple[str, Optional[str]]]:
-        # async context manager is used to prevent a message about
-        # unclosed session
-        async with await root.init_client() as client:
-            presets = list(client.config.presets)
-            return [(p, None) for p in presets if p.startswith(incomplete)]
-
-
-PRESET = PresetType()
-
-
 def do_deprecated_quiet(
     ctx: click.Context, param: Union[click.Option, click.Parameter], value: Any
 ) -> Any:
@@ -700,7 +549,6 @@ deprecated_quiet_option: Any = option(
     is_eager=True,
 )
 
-
 if sys.version_info >= (3, 7):  # pragma: no cover
     from contextlib import AsyncExitStack  # noqa
 else:
@@ -708,7 +556,7 @@ else:
 
 
 def format_size(value: float) -> str:
-    return humanize.naturalsize(value, gnu=True, format="%.4g")
+    return humanize.naturalsize(value, gnu=True)
 
 
 def pager_maybe(
@@ -735,6 +583,8 @@ def pager_maybe(
 def steal_config_maybe(dst_path: pathlib.Path) -> None:
     if NEURO_STEAL_CONFIG in os.environ:
         src = pathlib.Path(os.environ[NEURO_STEAL_CONFIG])
+        if not src.exists():
+            return
         dst = Factory(dst_path).path
         dst.mkdir(mode=0o700)
         for f in src.iterdir():
