@@ -82,6 +82,15 @@ class TestImageParser:
         with pytest.raises(ValueError, match="too many tags"):
             self.parser.has_tag(image)
 
+    def test_has_tag_lstrip(self) -> None:
+        image = "image:game:latest"
+        assert self.parser.has_tag(image)
+        image = "image:game:mega"
+        assert self.parser.has_tag(image)
+        image = "image:game:v2.0:latest"
+        with pytest.raises(ValueError, match="too many tags"):
+            assert self.parser.has_tag(image)
+
     @pytest.mark.parametrize(
         "registry_url",
         ["http://reg.neu.ro", "https://reg.neu.ro", "https://reg.neu.ro/bla/bla"],
@@ -287,7 +296,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_with_user_with_tag(self) -> None:
         image = "image://other-cluster/bob/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
             owner="bob",
@@ -298,7 +307,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_with_user_with_tag_2(self) -> None:
         image = "image://other-cluster/bob/library/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="bob",
@@ -309,7 +318,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_with_user_no_tag(self) -> None:
         image = "image://other-cluster/bob/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
             owner="bob",
@@ -320,7 +329,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_with_user_no_tag_2(self) -> None:
         image = "image://other-cluster/bob/library/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="latest",
             owner="bob",
@@ -331,7 +340,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_no_slash_no_user_no_tag(self) -> None:
         image = "image:ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
             owner="alice",
@@ -342,7 +351,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_no_slash_no_user_no_tag_2(self) -> None:
         image = "image:library/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="latest",
             owner="alice",
@@ -353,7 +362,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_no_slash_no_user_with_tag(self) -> None:
         image = "image:ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
             owner="alice",
@@ -364,7 +373,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_with_scheme_no_slash_no_user_with_tag_2(self) -> None:
         image = "image:library/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="alice",
@@ -372,188 +381,196 @@ class TestImageParser:
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_1_slash_no_user_no_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_no_name_no_tag(
+        self,
+    ) -> None:
         image = "image:/ubuntu"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_no_tag(self) -> None:
+        image = "image:/bob/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_1_slash_no_user_no_tag_2(self) -> None:
-        image = "image:/library/ubuntu"
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_no_tag_2(self) -> None:
+        image = "image:/bob/library/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="latest",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_1_slash_no_user_with_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_no_name_with_tag(
+        self,
+    ) -> None:
         image = "image:/ubuntu:v10.04"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_with_tag(self) -> None:
+        image = "image:/bob/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_1_slash_no_user_with_tag_2(self) -> None:
-        image = "image:/library/ubuntu:v10.04"
+    def test_parse_as_neuro_image_with_scheme_1_slash_no_cluster_with_tag_2(
+        self,
+    ) -> None:
+        image = "image:/bob/library/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_2_slash_user_no_tag_fail(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_2_slash_cluster_user_no_name_no_tag_fail(
+        self,
+    ) -> None:
         image = "image://other-cluster/ubuntu"
         with pytest.raises(ValueError, match="no image name specified"):
             self.parser.parse_as_neuro_image(image)
 
-    def test_parse_as_neuro_image_with_scheme_2_slash_user_with_tag_fail(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_2_slash_cluster_no_user_with_tag_fail(
+        self,
+    ) -> None:
         image = "image://ubuntu:v10.04"
         with pytest.raises(ValueError, match="port can't be converted to integer"):
             self.parser.parse_as_neuro_image(image)
 
-    def test_parse_as_neuro_image_with_scheme_3_slash_no_user_no_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_no_name_no_tag(
+        self,
+    ) -> None:
         image = "image:///ubuntu"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_no_tag(self) -> None:
+        image = "image:///bob/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_3_slash_no_user_no_tag_2(self) -> None:
-        image = "image:///library/ubuntu"
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_no_tag_2(self) -> None:
+        image = "image:///bob/library/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="latest",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_3_slash_no_user_with_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_no_name_with_tag(
+        self,
+    ) -> None:
         image = "image:///ubuntu:v10.04"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_with_tag(self) -> None:
+        image = "image:///bob/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_3_slash_no_user_with_tag_2(self) -> None:
-        image = "image:///library/ubuntu:v10.04"
+    def test_parse_as_neuro_image_with_scheme_3_slash_no_cluster_with_tag_2(
+        self,
+    ) -> None:
+        image = "image:///bob/library/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_4_slash_no_user_with_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_no_name_with_tag(
+        self,
+    ) -> None:
         image = "image:////ubuntu:v10.04"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_with_tag(self) -> None:
+        image = "image:////bob/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_4_slash_no_user_with_tag_2(self) -> None:
-        image = "image:////library/ubuntu:v10.04"
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_with_tag_2(
+        self,
+    ) -> None:
+        image = "image:////bob/library/ubuntu:v10.04"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_4_slash_no_user_no_tag(self) -> None:
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_no_name_no_tag(
+        self,
+    ) -> None:
         image = "image:////ubuntu"
+        with pytest.raises(ValueError, match="no image name specified"):
+            self.parser.parse_as_neuro_image(image)
+
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_no_tag(self) -> None:
+        image = "image:////bob/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
 
-    def test_parse_as_neuro_image_with_scheme_4_slash_no_user_no_tag_2(self) -> None:
-        image = "image:////library/ubuntu"
+    def test_parse_as_neuro_image_with_scheme_4_slash_no_cluster_no_tag_2(self) -> None:
+        image = "image:////bob/library/ubuntu"
         parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="latest",
-            owner="alice",
-            cluster_name="test-cluster",
-            registry="reg.neu.ro",
-        )
-
-    def test_parse_as_neuro_image_with_scheme_tilde_user_no_tag(self) -> None:
-        image = "image:ubuntu"
-        parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
-            name="ubuntu",
-            tag="latest",
-            owner="alice",
-            cluster_name="test-cluster",
-            registry="reg.neu.ro",
-        )
-
-    def test_parse_as_neuro_image_with_scheme_tilde_user_no_tag_2(self) -> None:
-        image = "image:library/ubuntu"
-        parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
-            name="library/ubuntu",
-            tag="latest",
-            owner="alice",
-            cluster_name="test-cluster",
-            registry="reg.neu.ro",
-        )
-
-    def test_parse_as_neuro_image_with_scheme_tilde_user_with_tag(self) -> None:
-        image = "image:ubuntu:v10.04"
-        parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
-            name="ubuntu",
-            tag="v10.04",
-            owner="alice",
-            cluster_name="test-cluster",
-            registry="reg.neu.ro",
-        )
-
-    def test_parse_as_neuro_image_with_scheme_tilde_user_with_tag_2(self) -> None:
-        image = "image:library/ubuntu:v10.04"
-        parsed = self.parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
-            name="library/ubuntu",
-            tag="v10.04",
-            owner="alice",
+            owner="bob",
             cluster_name="test-cluster",
             registry="reg.neu.ro",
         )
@@ -609,7 +626,7 @@ class TestImageParser:
     def test_parse_as_neuro_image_allow_tag_false_with_scheme_no_tag(self) -> None:
         image = "image:ubuntu"
         parsed = self.parser.parse_as_neuro_image(image, tag_option=TagOption.DENY)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag=None,
             owner="alice",
@@ -627,8 +644,19 @@ class TestImageParser:
         with pytest.raises(ValueError, match="tag is not allowed"):
             self.parser.parse_as_neuro_image(image, tag_option=TagOption.DENY)
 
+    def test_parse_as_neuro_image_allow_tag_false_with_scheme_lstrip(self) -> None:
+        image = "image:game:latest"
+        with pytest.raises(ValueError, match="tag is not allowed"):
+            self.parser.parse_as_neuro_image(image, tag_option=TagOption.DENY)
+        image = "image:game:mega"
+        with pytest.raises(ValueError, match="tag is not allowed"):
+            self.parser.parse_as_neuro_image(image, tag_option=TagOption.DENY)
+        image = "image:game:v2.0:latest"
+        with pytest.raises(ValueError, match="too many tags"):
+            self.parser.parse_as_neuro_image(image, tag_option=TagOption.DENY)
+
     def test_convert_to_local_image(self) -> None:
-        neuro_image = RemoteImage(
+        neuro_image = RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
             owner="artem",
@@ -641,7 +669,7 @@ class TestImageParser:
     def test_convert_to_neuro_image(self) -> None:
         local_image = LocalImage(name="ubuntu", tag="latest")
         neuro_image = self.parser.convert_to_neuro_image(local_image)
-        assert neuro_image == RemoteImage(
+        assert neuro_image == RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="latest",
             owner="alice",
@@ -731,7 +759,7 @@ class TestImageParser:
         )
         image = "image://test-cluster/bob/library/ubuntu:v10.04"
         parsed = my_parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="bob",
@@ -747,7 +775,7 @@ class TestImageParser:
         )
         image = "localhost:5000/bob/library/ubuntu:v10.04"
         parsed = my_parser.parse_as_neuro_image(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="bob",
@@ -773,7 +801,7 @@ class TestImageParser:
         )
         image = "image://test-cluster/bob/library/ubuntu:v10.04"
         parsed = my_parser.parse_remote(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="bob",
@@ -789,7 +817,7 @@ class TestImageParser:
         )
         image = "localhost:5000/bob/library/ubuntu:v10.04"
         parsed = my_parser.parse_remote(image)
-        assert parsed == RemoteImage(
+        assert parsed == RemoteImage.new_neuro_image(
             name="library/ubuntu",
             tag="v10.04",
             owner="bob",
@@ -806,18 +834,14 @@ class TestImageParser:
         image = "example.com:9999/bob/library/ubuntu:v10.04"
         parsed = my_parser.parse_remote(image)
         # NOTE: "owner" is parsed only for images in neuromation registry
-        assert parsed == RemoteImage(
-            name="bob/library/ubuntu",
-            tag="v10.04",
-            owner=None,
-            cluster_name=None,
-            registry="example.com:9999",
+        assert parsed == RemoteImage.new_external_image(
+            name="bob/library/ubuntu", tag="v10.04", registry="example.com:9999",
         )
 
 
 class TestRemoteImage:
     def test_as_str_in_neuro_registry_tag_none(self) -> None:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             name="ubuntu",
             tag=None,
             owner="me",
@@ -828,7 +852,7 @@ class TestRemoteImage:
         assert _as_repo_str(image) == "registry.io/me/ubuntu"
 
     def test_as_str_in_neuro_registry_tag_yes(self) -> None:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             name="ubuntu",
             tag="v10.04",
             owner="me",
@@ -839,15 +863,29 @@ class TestRemoteImage:
         assert _as_repo_str(image) == "registry.io/me/ubuntu:v10.04"
 
     def test_as_str_not_in_neuro_registry_tag_none(self) -> None:
-        image = RemoteImage(name="ubuntu", tag=None, owner=None, registry=None)
+        image = RemoteImage.new_external_image(name="ubuntu")
         assert str(image) == "ubuntu"
         assert _as_repo_str(image) == "ubuntu"
 
     def test_as_str_not_in_neuro_registry_tag_yes(self) -> None:
-        image = RemoteImage(name="ubuntu", tag="v10.04", owner=None, registry=None)
+        image = RemoteImage.new_external_image(name="ubuntu", tag="v10.04")
         assert str(image) == "ubuntu:v10.04"
         assert _as_repo_str(image) == "ubuntu:v10.04"
 
+    def test_as_docker_url_in_neuro_registry(self) -> None:
+        image = RemoteImage(
+            name="ubuntu",
+            tag="v10.04",
+            owner="me",
+            cluster_name="test-cluster",
+            registry="registry.io",
+        )
+        assert image.as_docker_url() == "registry.io/me/ubuntu:v10.04"
+
+    def test_as_docker_url_not_in_neuro_registry(self) -> None:
+        image = RemoteImage(name="ubuntu", tag="v10.04", owner=None, registry=None)
+        assert image.as_docker_url() == "ubuntu:v10.04"
+
 
 @pytest.mark.usefixtures("patch_docker_host")
 class TestImages:
@@ -1073,15 +1111,16 @@ class TestRegistry:
             ret = await client.images.ls()
 
         registry = _get_url_authority(registry_url)
+        assert registry is not None
         assert set(ret) == {
-            RemoteImage(
+            RemoteImage.new_neuro_image(
                 "alpine",
                 tag=None,
                 owner="bob",
                 cluster_name="default",
                 registry=registry,
             ),
-            RemoteImage(
+            RemoteImage.new_neuro_image(
                 "bananas",
                 tag=None,
                 owner="jill",
@@ -1097,7 +1136,7 @@ class TestRegistry:
         url = URL("http://whatever")
         registry_url = URL("http://whatever-registry")
         async with make_client(url, registry_url=registry_url) as client:
-            image = RemoteImage(
+            image = RemoteImage.new_neuro_image(
                 name="ubuntu",
                 tag="latest",
                 owner="me",
@@ -1107,17 +1146,6 @@ class TestRegistry:
             with pytest.raises(ValueError, match="tag is not allowed"):
                 await client.images.tags(image)
 
-    @pytest.mark.skipif(
-        sys.platform == "win32", reason="aiodocker doesn't support Windows pipes yet"
-    )
-    async def test_tags_bad_image_without_owner(self, make_client: _MakeClient) -> None:
-        url = URL("http://whatever")
-        registry_url = URL("http://whatever-registry")
-        async with make_client(url, registry_url=registry_url) as client:
-            image = RemoteImage(name="ubuntu", tag=None, owner=None, registry="reg")
-            with pytest.raises(ValueError, match="missing image owner"):
-                await client.images.tags(image)
-
     @pytest.mark.skipif(
         sys.platform == "win32", reason="aiodocker doesn't support Windows pipes yet"
     )
@@ -1125,7 +1153,7 @@ class TestRegistry:
         url = URL("http://whatever")
         registry_url = URL("http://whatever-registry")
         async with make_client(url, registry_url=registry_url) as client:
-            image = RemoteImage(
+            image = RemoteImage.new_neuro_image(
                 name="",
                 tag=None,
                 owner="me",
