@@ -4,12 +4,14 @@ from typing import Any, Callable, Dict, List, Optional
 import pytest
 from aiodocker.exceptions import DockerError
 from aiohttp import web
+from dateutil.parser import isoparse
 from yarl import URL
 
 from neuromation.api import (
     Client,
     Container,
     HTTPPort,
+    JobRestartPolicy,
     JobStatus,
     JobTelemetry,
     RemoteImage,
@@ -210,7 +212,7 @@ async def test_kill_ok(
 
 async def test_save_image_not_in_neuro_registry(make_client: _MakeClient) -> None:
     async with make_client("http://whatever") as client:
-        image = RemoteImage(name="ubuntu")
+        image = RemoteImage.new_external_image(name="ubuntu")
         with pytest.raises(ValueError, match="must be in the neuromation registry"):
             await client.jobs.save("job-id", image)
 
@@ -259,7 +261,7 @@ async def test_save_ok(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         await client.jobs.save("job-id", image)
@@ -294,7 +296,7 @@ async def test_save_commit_started_invalid_status_fails(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         with pytest.raises(
@@ -332,7 +334,7 @@ async def test_save_commit_started_missing_image_details_fails(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         with pytest.raises(DockerError, match="Missing required details: 'image'"):
@@ -367,7 +369,7 @@ async def test_save_commit_finished_invalid_status_fails(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         with pytest.raises(
@@ -404,7 +406,7 @@ async def test_save_commit_started_missing_status_fails(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         with pytest.raises(DockerError, match='Missing required field: "status"'):
@@ -438,7 +440,7 @@ async def test_save_commit_finished_missing_status_fails(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        image = RemoteImage(
+        image = RemoteImage.new_neuro_image(
             registry="gcr.io", owner="me", cluster_name="test-cluster", name="img"
         )
         with pytest.raises(DockerError, match='Missing required field: "status"'):
@@ -465,6 +467,7 @@ async def test_status_failed(
         "is_preemptible": True,
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-id",
         "container": {
             "image": "submit-image-name",
             "command": "submit-command",
@@ -525,6 +528,7 @@ async def test_status_with_ssh_and_http(
         "is_preemptible": True,
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-id",
         "container": {
             "image": "submit-image-name",
             "command": "submit-command",
@@ -585,6 +589,7 @@ async def test_status_with_tpu(
         "is_preemptible": True,
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-id",
         "container": {
             "image": "submit-image-name",
             "command": "submit-command",
@@ -644,6 +649,7 @@ async def test_job_run(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -712,7 +718,7 @@ async def test_job_run(
             ),
         ]
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             volumes=volumes,
@@ -741,6 +747,7 @@ async def test_job_run_with_name_and_description(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -811,7 +818,7 @@ async def test_job_run_with_name_and_description(
             ),
         ]
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             volumes=volumes,
@@ -844,6 +851,7 @@ async def test_job_run_with_tags(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -913,7 +921,7 @@ async def test_job_run_with_tags(
             ),
         ]
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             volumes=volumes,
@@ -943,6 +951,7 @@ async def test_job_run_no_volumes(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -991,7 +1000,7 @@ async def test_job_run_no_volumes(
     async with make_client(srv.make_url("/")) as client:
         resources = Resources(16384, 7, 1, "test-gpu-model", True, None, None)
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             http=HTTPPort(8181),
@@ -1023,6 +1032,7 @@ async def test_job_run_preemptible(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -1093,7 +1103,7 @@ async def test_job_run_preemptible(
             ),
         ]
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             volumes=volumes,
@@ -1125,6 +1135,7 @@ async def test_job_run_schedule_timeout(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -1172,7 +1183,7 @@ async def test_job_run_schedule_timeout(
     async with make_client(srv.make_url("/")) as client:
         resources = Resources(16384, 7, 1, "test-gpu-model", True, None, None)
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
         )
@@ -1197,6 +1208,7 @@ async def test_job_run_tpu(
         },
         "owner": "owner",
         "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
         "container": {
             "image": "gcr.io/light-reality-205619/ubuntu:latest",
             "command": "date",
@@ -1245,7 +1257,7 @@ async def test_job_run_tpu(
     async with make_client(srv.make_url("/")) as client:
         resources = Resources(16384, 7, 1, "test-gpu-model", True, "v3-8", "1.14")
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
         )
@@ -1285,7 +1297,7 @@ async def test_job_run_with_tty(
     async with make_client(srv.make_url("/")) as client:
         resources = Resources(16384, 0.5)
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
             tty=True,
@@ -1293,6 +1305,7 @@ async def test_job_run_with_tty(
         ret = await client.jobs.run(container=container)
 
         assert ret == _job_description_from_api(JSON, client.parse)
+        assert ret.container.tty is True
 
 
 def create_job_response(
@@ -1328,6 +1341,7 @@ def create_job_response(
         "is_preemptible": True,
         "owner": owner,
         "cluster_name": "default",
+        "uri": f"job://default/{owner}/{id}",
     }
     if name:
         result["name"] = name
@@ -1355,7 +1369,7 @@ async def test_list_no_filter(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list()
+        ret = [job async for job in client.jobs.list()]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1393,7 +1407,7 @@ async def test_list_filter_by_name(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(name=name_1)
+        ret = [job async for job in client.jobs.list(name=name_1)]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1432,7 +1446,7 @@ async def test_list_filter_by_statuses(
 
     statuses = {JobStatus.FAILED, JobStatus.SUCCEEDED}
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(statuses=statuses)
+        ret = [job async for job in client.jobs.list(statuses=statuses)]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1464,7 +1478,7 @@ async def test_list_incorrect_image(
 
     statuses = {JobStatus.FAILED, JobStatus.SUCCEEDED}
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(statuses=statuses)
+        ret = [job async for job in client.jobs.list(statuses=statuses)]
     for job in ret:
         if job.status == JobStatus.FAILED:
             assert job.container.image.name == INVALID_IMAGE_NAME
@@ -1589,7 +1603,7 @@ async def test_list_filter_by_name_and_statuses(
     statuses = {JobStatus.PENDING, JobStatus.SUCCEEDED}
     name = "job-name-1"
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(statuses=statuses, name=name)
+        ret = [job async for job in client.jobs.list(statuses=statuses, name=name)]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1626,7 +1640,7 @@ async def test_list_filter_by_tags(
 
     tags = {"t1", "t2"}
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(tags=tags)
+        ret = [job async for job in client.jobs.list(tags=tags)]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1673,7 +1687,12 @@ async def test_list_filter_by_name_and_statuses_and_owners(
     name = name_1
     owners = {owner_1, owner_2}
     async with make_client(srv.make_url("/")) as client:
-        ret = await client.jobs.list(statuses=statuses, name=name, owners=owners)
+        ret = [
+            job
+            async for job in client.jobs.list(
+                statuses=statuses, name=name, owners=owners
+            )
+        ]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1681,6 +1700,60 @@ async def test_list_filter_by_name_and_statuses_and_owners(
         assert ret == job_descriptions[:2]
 
 
+async def test_list_filter_by_date_range(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    jobs = [
+        create_job_response("job-id-1", "running"),
+        create_job_response("job-id-2", "running"),
+        create_job_response("job-id-3", "running"),
+    ]
+    jobs[0]["history"]["created_at"] = "2018-09-25T12:28:21.298672+00:00"
+    jobs[1]["history"]["created_at"] = "2018-09-25T12:28:26.698687+00:00"
+    jobs[2]["history"]["created_at"] = "2018-09-25T12:28:31.642202+00:00"
+
+    t1 = isoparse(jobs[0]["history"]["created_at"])
+    t2 = isoparse(jobs[1]["history"]["created_at"])
+    t3 = isoparse(jobs[2]["history"]["created_at"])
+    t2naive = t2.astimezone(None).replace(tzinfo=None)
+
+    async def handler(request: web.Request) -> web.Response:
+        since = isoparse(request.query.get("since", "0001-01-01T00:00:00+00:00"))
+        until = isoparse(request.query.get("until", "9999-12-31T23:59:59.999999+00:00"))
+        filtered_jobs = [
+            job
+            for job in jobs
+            if since <= isoparse(job["history"]["created_at"]) <= until
+        ]
+        return web.json_response({"jobs": filtered_jobs})
+
+    app = web.Application()
+    app.router.add_get("/jobs", handler)
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        ret = [job async for job in client.jobs.list()]
+        assert {job.id for job in ret} == {"job-id-1", "job-id-2", "job-id-3"}
+
+        ret = [job async for job in client.jobs.list(since=t2)]
+        assert {job.id for job in ret} == {"job-id-2", "job-id-3"}
+
+        ret = [job async for job in client.jobs.list(until=t2)]
+        assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
+
+        ret = [job async for job in client.jobs.list(since=t1, until=t2)]
+        assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
+
+        ret = [job async for job in client.jobs.list(since=t1, until=t3)]
+        assert {job.id for job in ret} == {"job-id-1", "job-id-2", "job-id-3"}
+
+        ret = [job async for job in client.jobs.list(since=t2naive)]
+        assert {job.id for job in ret} == {"job-id-2", "job-id-3"}
+
+        ret = [job async for job in client.jobs.list(until=t2naive)]
+        assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
+
+
 async def test_job_run_life_span(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -1706,8 +1779,42 @@ async def test_job_run_life_span(
     async with make_client(srv.make_url("/")) as client:
         resources = Resources(16, 0.5)
         container = Container(
-            image=RemoteImage("submit-image-name"),
+            image=RemoteImage.new_external_image(name="submit-image-name"),
             command="submit-command",
             resources=resources,
         )
         await client.jobs.run(container=container, life_span=10 * 60)
+
+
+async def test_job_run_restart_policy(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.Response:
+        data = await request.json()
+        assert data == {
+            "container": {
+                "image": "submit-image-name",
+                "resources": {"memory_mb": 16, "cpu": 0.5, "shm": True},
+                "command": "submit-command",
+            },
+            "is_preemptible": False,
+            "restart_policy": "always",
+            "cluster_name": "default",
+        }
+        return web.json_response(create_job_response("job-id-1", "running"))
+
+    app = web.Application()
+    app.router.add_post("/jobs", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        resources = Resources(16, 0.5)
+        container = Container(
+            image=RemoteImage.new_external_image(name="submit-image-name"),
+            command="submit-command",
+            resources=resources,
+        )
+        await client.jobs.run(
+            container=container, restart_policy=JobRestartPolicy.ALWAYS
+        )
