@@ -11,6 +11,7 @@ from neuromation.api import (
     Container,
     HTTPPort,
     JobDescription,
+    JobRestartPolicy,
     JobStatus,
     JobStatusHistory,
     JobTelemetry,
@@ -27,7 +28,12 @@ from neuromation.cli.formatters import (
     SimpleJobsFormatter,
     TabularJobsFormatter,
 )
-from neuromation.cli.formatters.jobs import ResourcesFormatter, TabularJobRow
+from neuromation.cli.formatters.jobs import (
+    ResourcesFormatter,
+    TabularJobRow,
+    format_timedelta,
+)
+from neuromation.cli.formatters.utils import image_formatter, uri_formatter
 from neuromation.cli.parse_utils import parse_columns
 from neuromation.cli.printer import CSI
 
@@ -43,6 +49,7 @@ def job_descr_no_name() -> JobDescription:
         id=TEST_JOB_ID,
         owner="owner",
         cluster_name="default",
+        uri=URL(f"job://default/owner/{TEST_JOB_ID}"),
         history=JobStatusHistory(
             status=JobStatus.PENDING,
             reason="ErrorReason",
@@ -52,7 +59,7 @@ def job_descr_no_name() -> JobDescription:
             finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
         ),
         container=Container(
-            image=RemoteImage("ubuntu", "latest"),
+            image=RemoteImage.new_external_image(name="ubuntu", tag="latest"),
             resources=Resources(16, 0.1, 0, None, False, None, None),
         ),
         ssh_server=URL("ssh-auth"),
@@ -68,6 +75,7 @@ def job_descr() -> JobDescription:
         name=TEST_JOB_NAME,
         owner="owner",
         cluster_name="default",
+        uri=URL(f"job://default/owner/{TEST_JOB_ID}"),
         history=JobStatusHistory(
             status=JobStatus.PENDING,
             reason="ErrorReason",
@@ -77,7 +85,7 @@ def job_descr() -> JobDescription:
             finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
         ),
         container=Container(
-            image=RemoteImage("ubuntu", "latest"),
+            image=RemoteImage.new_external_image(name="ubuntu", tag="latest"),
             resources=Resources(16, 0.1, 0, None, False, None, None),
         ),
         ssh_server=URL("ssh-auth"),
@@ -171,6 +179,7 @@ class TestJobStartProgress:
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             description="test job description",
             http_url=URL("http://local.host.test/"),
             history=JobStatusHistory(
@@ -183,7 +192,7 @@ class TestJobStartProgress:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
             ssh_server=URL("ssh-auth"),
@@ -238,6 +247,7 @@ class TestJobOutputFormatter:
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             name="test-job-name",
             description="test job description",
             http_url=URL("http://local.host.test/"),
@@ -252,7 +262,7 @@ class TestJobOutputFormatter:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 http=HTTPPort(port=80, requires_auth=True),
             ),
@@ -260,7 +270,8 @@ class TestJobOutputFormatter:
             is_preemptible=False,
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -289,6 +300,7 @@ class TestJobOutputFormatter:
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             tags=["tag1", "tag2", "tag3"],
             description="test job description",
             http_url=URL("http://local.host.test/"),
@@ -303,7 +315,7 @@ class TestJobOutputFormatter:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 http=HTTPPort(port=80, requires_auth=True),
             ),
@@ -311,7 +323,8 @@ class TestJobOutputFormatter:
             is_preemptible=False,
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -334,12 +347,172 @@ class TestJobOutputFormatter:
             "ErrorDesc\n================="
         )
 
+    def test_job_with_life_span_with_value(self) -> None:
+        description = JobDescription(
+            status=JobStatus.FAILED,
+            owner="test-user",
+            cluster_name="default",
+            id="test-job",
+            uri=URL("job://default/test-user/test-job"),
+            description="test job description",
+            http_url=URL("http://local.host.test/"),
+            history=JobStatusHistory(
+                status=JobStatus.PENDING,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                exit_code=123,
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_external_image(name="test-image"),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+                http=HTTPPort(port=80, requires_auth=True),
+            ),
+            ssh_server=URL("ssh-auth"),
+            is_preemptible=False,
+            life_span=1.0 * ((60 * 60 * 24 * 1) + (60 * 60 * 2) + (60 * 3) + 4),
+        )
+
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
+        resource_formatter = ResourcesFormatter()
+        assert (
+            status == "Job: test-job\n"
+            "Owner: test-user\n"
+            "Cluster: default\n"
+            "Description: test job description\n"
+            "Status: failed (ErrorReason)\n"
+            "Image: test-image\n"
+            "Command: test-command\n"
+            f"{resource_formatter(description.container.resources)}\n"
+            "Preemptible: False\n"
+            "Life span: 1d2h3m4s\n"
+            "Http URL: http://local.host.test/\n"
+            "Http authentication: True\n"
+            "Created: 2018-09-25T12:28:21.298672+00:00\n"
+            "Started: 2018-09-25T12:28:59.759433+00:00\n"
+            "Finished: 2018-09-25T12:28:59.759433+00:00\n"
+            "Exit code: 123\n"
+            "===Description===\n"
+            "ErrorDesc\n================="
+        )
+
+    def test_job_with_life_span_without_value(self) -> None:
+        description = JobDescription(
+            status=JobStatus.FAILED,
+            owner="test-user",
+            cluster_name="default",
+            id="test-job",
+            uri=URL("job://default/test-user/test-job"),
+            description="test job description",
+            http_url=URL("http://local.host.test/"),
+            history=JobStatusHistory(
+                status=JobStatus.PENDING,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                exit_code=123,
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_external_image(name="test-image"),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+                http=HTTPPort(port=80, requires_auth=True),
+            ),
+            ssh_server=URL("ssh-auth"),
+            is_preemptible=False,
+            life_span=0.0,
+        )
+
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
+        resource_formatter = ResourcesFormatter()
+        assert (
+            status == "Job: test-job\n"
+            "Owner: test-user\n"
+            "Cluster: default\n"
+            "Description: test job description\n"
+            "Status: failed (ErrorReason)\n"
+            "Image: test-image\n"
+            "Command: test-command\n"
+            f"{resource_formatter(description.container.resources)}\n"
+            "Preemptible: False\n"
+            "Life span: no limit\n"
+            "Http URL: http://local.host.test/\n"
+            "Http authentication: True\n"
+            "Created: 2018-09-25T12:28:21.298672+00:00\n"
+            "Started: 2018-09-25T12:28:59.759433+00:00\n"
+            "Finished: 2018-09-25T12:28:59.759433+00:00\n"
+            "Exit code: 123\n"
+            "===Description===\n"
+            "ErrorDesc\n================="
+        )
+
+    def test_job_with_restart_policy(self) -> None:
+        description = JobDescription(
+            status=JobStatus.FAILED,
+            owner="test-user",
+            cluster_name="default",
+            id="test-job",
+            uri=URL("job://default/test-user/test-job"),
+            description="test job description",
+            http_url=URL("http://local.host.test/"),
+            history=JobStatusHistory(
+                status=JobStatus.PENDING,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                exit_code=123,
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_external_image(name="test-image"),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+                http=HTTPPort(port=80, requires_auth=True),
+            ),
+            ssh_server=URL("ssh-auth"),
+            is_preemptible=False,
+            restart_policy=JobRestartPolicy.ALWAYS,
+        )
+
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
+        resource_formatter = ResourcesFormatter()
+        assert (
+            status == "Job: test-job\n"
+            "Owner: test-user\n"
+            "Cluster: default\n"
+            "Description: test job description\n"
+            "Status: failed (ErrorReason)\n"
+            "Image: test-image\n"
+            "Command: test-command\n"
+            f"{resource_formatter(description.container.resources)}\n"
+            "Preemptible: False\n"
+            "Restart policy: always\n"
+            "Http URL: http://local.host.test/\n"
+            "Http authentication: True\n"
+            "Created: 2018-09-25T12:28:21.298672+00:00\n"
+            "Started: 2018-09-25T12:28:59.759433+00:00\n"
+            "Finished: 2018-09-25T12:28:59.759433+00:00\n"
+            "Exit code: 123\n"
+            "===Description===\n"
+            "ErrorDesc\n================="
+        )
+
     def test_pending_job(self) -> None:
         description = JobDescription(
             status=JobStatus.FAILED,
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             description="test job description",
             http_url=URL("http://local.host.test/"),
             history=JobStatusHistory(
@@ -353,7 +526,7 @@ class TestJobOutputFormatter:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 http=HTTPPort(port=80, requires_auth=True),
             ),
@@ -361,7 +534,8 @@ class TestJobOutputFormatter:
             is_preemptible=False,
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -398,16 +572,18 @@ class TestJobOutputFormatter:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
             ssh_server=URL("ssh-auth"),
             is_preemptible=True,
             owner="owner",
             cluster_name="default",
+            uri=URL("job://default/owner/test-job"),
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -436,7 +612,7 @@ class TestJobOutputFormatter:
                 finished_at=None,
             ),
             container=Container(
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 command="test-command",
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
@@ -444,9 +620,11 @@ class TestJobOutputFormatter:
             is_preemptible=True,
             owner="owner",
             cluster_name="default",
+            uri=URL("job://default/owner/test-job"),
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -475,7 +653,7 @@ class TestJobOutputFormatter:
                 finished_at=None,
             ),
             container=Container(
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 command="test-command",
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
@@ -483,9 +661,11 @@ class TestJobOutputFormatter:
             is_preemptible=True,
             owner="owner",
             cluster_name="default",
+            uri=URL("job://default/owner/test-job"),
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -505,6 +685,7 @@ class TestJobOutputFormatter:
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             description="test job description",
             history=JobStatusHistory(
                 status=JobStatus.RUNNING,
@@ -517,7 +698,7 @@ class TestJobOutputFormatter:
             http_url=URL("http://local.host.test/"),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
             ssh_server=URL("ssh-auth"),
@@ -525,7 +706,8 @@ class TestJobOutputFormatter:
             internal_hostname="host.local",
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -549,6 +731,7 @@ class TestJobOutputFormatter:
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             description="test job description",
             history=JobStatusHistory(
                 status=JobStatus.RUNNING,
@@ -562,7 +745,7 @@ class TestJobOutputFormatter:
             container=Container(
                 entrypoint="/usr/bin/make",
                 command="test",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_external_image(name="test-image"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
             ),
             ssh_server=URL("ssh-auth"),
@@ -570,7 +753,8 @@ class TestJobOutputFormatter:
             internal_hostname="host.local",
         )
 
-        status = JobStatusFormatter()(description)
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -589,12 +773,13 @@ class TestJobOutputFormatter:
             "Started: 2018-09-25T12:28:24.759433+00:00"
         )
 
-    def test_job_with_volumes(self) -> None:
+    def test_job_with_volumes_short(self) -> None:
         description = JobDescription(
             status=JobStatus.FAILED,
             owner="test-user",
             cluster_name="default",
             id="test-job",
+            uri=URL("job://default/test-user/test-job"),
             name="test-job-name",
             description="test job description",
             http_url=URL("http://local.host.test/"),
@@ -609,12 +794,98 @@ class TestJobOutputFormatter:
             ),
             container=Container(
                 command="test-command",
-                image=RemoteImage("test-image"),
+                image=RemoteImage.new_neuro_image(
+                    name="test-image",
+                    tag="sometag",
+                    registry="https://registry.neu.ro",
+                    owner="test-user",
+                    cluster_name="test-cluster",
+                ),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 http=HTTPPort(port=80, requires_auth=True),
                 volumes=[
                     Volume(
-                        storage_uri=URL("storage://test-cluster/test-user/ro"),
+                        storage_uri=URL("storage://test-cluster/otheruser/ro"),
+                        container_path="/mnt/ro",
+                        read_only=True,
+                    ),
+                    Volume(
+                        storage_uri=URL("storage://test-cluster/test-user/rw"),
+                        container_path="/mnt/rw",
+                        read_only=False,
+                    ),
+                    Volume(
+                        storage_uri=URL("storage://othercluster/otheruser/ro"),
+                        container_path="/mnt/ro2",
+                        read_only=True,
+                    ),
+                ],
+            ),
+            ssh_server=URL("ssh-auth"),
+            is_preemptible=False,
+        )
+
+        uri_fmtr = uri_formatter(username="test-user", cluster_name="test-cluster")
+        status = JobStatusFormatter(uri_formatter=uri_fmtr)(description)
+        resource_formatter = ResourcesFormatter()
+        assert (
+            status == "Job: test-job\n"
+            "Name: test-job-name\n"
+            "Owner: test-user\n"
+            "Cluster: default\n"
+            "Description: test job description\n"
+            "Status: failed (ErrorReason)\n"
+            "Image: image:test-image:sometag\n"
+            "Command: test-command\n"
+            f"{resource_formatter(description.container.resources)}\n"
+            "Preemptible: False\n"
+            "Volumes:\n"
+            "  /mnt/ro   storage:/otheruser/ro                READONLY\n"
+            "  /mnt/rw   storage:rw                                   \n"
+            "  /mnt/ro2  storage://othercluster/otheruser/ro  READONLY\n"
+            "Http URL: http://local.host.test/\n"
+            "Http authentication: True\n"
+            "Created: 2018-09-25T12:28:21.298672+00:00\n"
+            "Started: 2018-09-25T12:28:59.759433+00:00\n"
+            "Finished: 2018-09-25T12:28:59.759433+00:00\n"
+            "Exit code: 123\n"
+            "===Description===\n"
+            "ErrorDesc\n================="
+        )
+
+    def test_job_with_volumes_long(self) -> None:
+        description = JobDescription(
+            status=JobStatus.FAILED,
+            owner="test-user",
+            cluster_name="default",
+            id="test-job",
+            uri=URL("job://default/test-user/test-job"),
+            name="test-job-name",
+            description="test job description",
+            http_url=URL("http://local.host.test/"),
+            history=JobStatusHistory(
+                status=JobStatus.PENDING,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                exit_code=123,
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_neuro_image(
+                    name="test-image",
+                    tag="sometag",
+                    registry="https://registry.neu.ro",
+                    owner="test-user",
+                    cluster_name="test-cluster",
+                ),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+                http=HTTPPort(port=80, requires_auth=True),
+                volumes=[
+                    Volume(
+                        storage_uri=URL("storage://test-cluster/otheruser/ro"),
                         container_path="/mnt/ro",
                         read_only=True,
                     ),
@@ -629,7 +900,7 @@ class TestJobOutputFormatter:
             is_preemptible=False,
         )
 
-        status = JobStatusFormatter()(description)
+        status = JobStatusFormatter(uri_formatter=str)(description)
         resource_formatter = ResourcesFormatter()
         assert (
             status == "Job: test-job\n"
@@ -638,12 +909,12 @@ class TestJobOutputFormatter:
             "Cluster: default\n"
             "Description: test job description\n"
             "Status: failed (ErrorReason)\n"
-            "Image: test-image\n"
+            "Image: image://test-cluster/test-user/test-image:sometag\n"
             "Command: test-command\n"
             f"{resource_formatter(description.container.resources)}\n"
             "Preemptible: False\n"
             "Volumes:\n"
-            "  /mnt/ro  storage://test-cluster/test-user/ro  READONLY\n"
+            "  /mnt/ro  storage://test-cluster/otheruser/ro  READONLY\n"
             "  /mnt/rw  storage://test-cluster/test-user/rw          \n"
             "Http URL: http://local.host.test/\n"
             "Http authentication: True\n"
@@ -713,6 +984,42 @@ class TestJobTelemetryFormatter:
         )
 
 
+class TestJobStatusFormatter:
+    def test_format_timedelta(self) -> None:
+        delta = timedelta(days=1, hours=2, minutes=3, seconds=4)
+        assert format_timedelta(delta) == "1d2h3m4s"
+
+    def test_format_timedelta_no_days(self) -> None:
+        delta = timedelta(hours=2, minutes=3, seconds=4)
+        assert format_timedelta(delta) == "2h3m4s"
+
+    def test_format_timedelta_no_hours(self) -> None:
+        delta = timedelta(days=1, minutes=3, seconds=4)
+        assert format_timedelta(delta) == "1d3m4s"
+
+    def test_format_timedelta_no_minutes(self) -> None:
+        delta = timedelta(days=1, hours=2, seconds=4)
+        assert format_timedelta(delta) == "1d2h4s"
+
+    def test_format_timedelta_no_seconds(self) -> None:
+        delta = timedelta(days=1, hours=2, minutes=3)
+        assert format_timedelta(delta) == "1d2h3m"
+
+    def test_format_timedelta_overfill(self) -> None:
+        minutes = 60 * 24 * 30 + 20
+        delta = timedelta(minutes=minutes, seconds=10)
+        assert format_timedelta(delta) == "30d20m10s"
+
+    def test_format_timedelta_zero(self) -> None:
+        delta = timedelta(0)
+        assert format_timedelta(delta) == ""
+
+    def test_format_timedelta_negative(self) -> None:
+        delta = timedelta(-1)
+        with pytest.raises(ValueError, match="Invalid delta"):
+            assert format_timedelta(delta)
+
+
 class TestSimpleJobsFormatter:
     def test_empty(self) -> None:
         formatter = SimpleJobsFormatter()
@@ -726,6 +1033,7 @@ class TestSimpleJobsFormatter:
                 id="job-42687e7c-6c76-4857-a6a7-1166f8295391",
                 owner="owner",
                 cluster_name="default",
+                uri=URL("job://default/owner/job-42687e7c-6c76-4857-a6a7-1166f8295391"),
                 history=JobStatusHistory(
                     status=JobStatus.PENDING,
                     reason="ErrorReason",
@@ -735,7 +1043,7 @@ class TestSimpleJobsFormatter:
                     finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
                 ),
                 container=Container(
-                    image=RemoteImage("ubuntu", "latest"),
+                    image=RemoteImage.new_external_image(name="ubuntu", tag="latest"),
                     resources=Resources(16, 0.1, 0, None, False, None, None),
                 ),
                 ssh_server=URL("ssh-auth"),
@@ -747,6 +1055,7 @@ class TestSimpleJobsFormatter:
                 name="this-job-has-a-name",
                 owner="owner",
                 cluster_name="default",
+                uri=URL("job://default/owner/job-cf33bd55-9e3b-4df7-a894-9c148a908a66"),
                 history=JobStatusHistory(
                     status=JobStatus.FAILED,
                     reason="ErrorReason",
@@ -756,7 +1065,7 @@ class TestSimpleJobsFormatter:
                     finished_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
                 ),
                 container=Container(
-                    image=RemoteImage("ubuntu", "latest"),
+                    image=RemoteImage.new_external_image(name="ubuntu", tag="latest"),
                     resources=Resources(16, 0.1, 0, None, False, None, None),
                 ),
                 ssh_server=URL("ssh-auth"),
@@ -786,6 +1095,7 @@ class TestTabularJobRow:
             name=name,
             owner="owner",
             cluster_name="default",
+            uri=URL("job://default/owner/job-1f5ab792-e534-4bb4-be56-8af1ce722692"),
             description="some",
             history=JobStatusHistory(
                 status=status,
@@ -806,13 +1116,17 @@ class TestTabularJobRow:
 
     def test_with_job_name(self) -> None:
         row = TabularJobRow.from_job(
-            self._job_descr_with_status(JobStatus.RUNNING, name="job-name"), "owner"
+            self._job_descr_with_status(JobStatus.RUNNING, name="job-name"),
+            "owner",
+            image_formatter=str,
         )
         assert row.name == "job-name"
 
     def test_without_job_name(self) -> None:
         row = TabularJobRow.from_job(
-            self._job_descr_with_status(JobStatus.RUNNING, name=None), "owner"
+            self._job_descr_with_status(JobStatus.RUNNING, name=None),
+            "owner",
+            image_formatter=str,
         )
         assert row.name == ""
 
@@ -826,16 +1140,32 @@ class TestTabularJobRow:
         ],
     )
     def test_status_date_relation(self, status: JobStatus, date: str) -> None:
-        row = TabularJobRow.from_job(self._job_descr_with_status(status), "owner")
+        row = TabularJobRow.from_job(
+            self._job_descr_with_status(status), "owner", image_formatter=str
+        )
         assert row.status == f"{status}"
         assert row.when == date
 
-    def test_image_from_registry_parsing(self) -> None:
+    def test_image_from_registry_parsing_short(self) -> None:
+        uri_fmtr = uri_formatter(username="bob", cluster_name="test-cluster")
+        image_fmtr = image_formatter(uri_formatter=uri_fmtr)
         row = TabularJobRow.from_job(
             self._job_descr_with_status(
-                JobStatus.PENDING, "registry-test.neu.ro/bob/swiss-box:red"
+                JobStatus.PENDING, "registry-test.neu.ro/bob/swiss-box:red",
+            ),
+            "bob",
+            image_formatter=image_fmtr,
+        )
+        assert row.image == "image:swiss-box:red"
+        assert row.name == ""
+
+    def test_image_from_registry_parsing_long(self) -> None:
+        row = TabularJobRow.from_job(
+            self._job_descr_with_status(
+                JobStatus.PENDING, "registry-test.neu.ro/bob/swiss-box:red",
             ),
             "owner",
+            image_formatter=str,
         )
         assert row.image == "image://test-cluster/bob/swiss-box:red"
         assert row.name == ""
@@ -858,12 +1188,16 @@ class TestTabularJobsFormatter:
     )
 
     def test_empty(self) -> None:
-        formatter = TabularJobsFormatter(0, "owner", parse_columns(None))
+        formatter = TabularJobsFormatter(
+            0, "owner", parse_columns(None), image_formatter=str
+        )
         result = [item for item in formatter([])]
         assert result == ["  ".join(self.columns)]
 
     def test_width_cutting(self) -> None:
-        formatter = TabularJobsFormatter(10, "owner", parse_columns(None))
+        formatter = TabularJobsFormatter(
+            10, "owner", parse_columns(None), image_formatter=str
+        )
         result = [item for item in formatter([])]
         assert result == ["  ".join(self.columns)[:10]]
 
@@ -876,6 +1210,7 @@ class TestTabularJobsFormatter:
             id="j",
             owner=owner_name,
             cluster_name="dc",
+            uri=URL(f"job://dc/{owner_name}/j"),
             name="name",
             description="d",
             history=JobStatusHistory(
@@ -887,14 +1222,16 @@ class TestTabularJobsFormatter:
                 finished_at=datetime.now(timezone.utc) - timedelta(seconds=1),
             ),
             container=Container(
-                image=RemoteImage("i", "l"),
+                image=RemoteImage.new_external_image(name="i", tag="l"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 command="c",
             ),
             ssh_server=URL("ssh-auth"),
             is_preemptible=True,
         )
-        formatter = TabularJobsFormatter(0, "owner", parse_columns(None))
+        formatter = TabularJobsFormatter(
+            0, "owner", parse_columns(None), image_formatter=str
+        )
         result = [item.rstrip() for item in formatter([job])]
         assert result in [
             [
@@ -922,6 +1259,10 @@ class TestTabularJobsFormatter:
                 name="name1",
                 owner=owner_name,
                 cluster_name="default",
+                uri=URL(
+                    f"job://default/{owner_name}/"
+                    f"job-7ee153a7-249c-4be9-965a-ba3eafb67c82"
+                ),
                 description="some description long long long long",
                 history=JobStatusHistory(
                     status=JobStatus.FAILED,
@@ -932,7 +1273,9 @@ class TestTabularJobsFormatter:
                     finished_at=isoparse("2017-09-25T12:28:59.759433+00:00"),
                 ),
                 container=Container(
-                    image=RemoteImage("some-image-name", "with-long-tag"),
+                    image=RemoteImage.new_external_image(
+                        name="some-image-name", tag="with-long-tag"
+                    ),
                     resources=Resources(16, 0.1, 0, None, False, None, None),
                     command="ls -la /some/path",
                 ),
@@ -945,6 +1288,10 @@ class TestTabularJobsFormatter:
                 name="name2",
                 owner=owner_name,
                 cluster_name="default",
+                uri=URL(
+                    f"job://default/{owner_name}/"
+                    f"job-7ee153a7-249c-4be9-965a-ba3eafb67c84"
+                ),
                 description="some description",
                 history=JobStatusHistory(
                     status=JobStatus.PENDING,
@@ -955,9 +1302,9 @@ class TestTabularJobsFormatter:
                     finished_at=isoparse("2017-09-25T12:28:59.759433+00:00"),
                 ),
                 container=Container(
-                    image=RemoteImage(
-                        "some-image-name",
-                        "with-long-tag",
+                    image=RemoteImage.new_neuro_image(
+                        name="some-image-name",
+                        tag="with-long-tag",
                         registry="https://registry.neu.ro",
                         owner="bob",
                         cluster_name="test-cluster",
@@ -969,7 +1316,9 @@ class TestTabularJobsFormatter:
                 is_preemptible=True,
             ),
         ]
-        formatter = TabularJobsFormatter(0, "owner", parse_columns(None))
+        formatter = TabularJobsFormatter(
+            0, "owner", parse_columns(None), image_formatter=str
+        )
         result = [item.rstrip() for item in formatter(jobs)]
         assert result == [
             f"ID                                        NAME   STATUS   WHEN         IMAGE                                     OWNER  CLUSTER  DESCRIPTION                           COMMAND",  # noqa: E501
@@ -984,6 +1333,7 @@ class TestTabularJobsFormatter:
             id="j",
             owner="owner",
             cluster_name="dc",
+            uri=URL("job://dc/owner/j"),
             name="name",
             description="d",
             history=JobStatusHistory(
@@ -995,7 +1345,7 @@ class TestTabularJobsFormatter:
                 finished_at=datetime.now(timezone.utc) - timedelta(seconds=1),
             ),
             container=Container(
-                image=RemoteImage("i", "l"),
+                image=RemoteImage.new_external_image(name="i", tag="l"),
                 resources=Resources(16, 0.1, 0, None, False, None, None),
                 command="c",
             ),
@@ -1004,7 +1354,7 @@ class TestTabularJobsFormatter:
         )
 
         columns = parse_columns("{status;align=right;min=20;Status Code}")
-        formatter = TabularJobsFormatter(0, "owner", columns)
+        formatter = TabularJobsFormatter(0, "owner", columns, image_formatter=str)
         result = [item.rstrip() for item in formatter([job])]
 
         assert result == ["         Status Code", "              failed"]
@@ -1024,7 +1374,7 @@ class TestResourcesFormatter:
         resource_formatter = ResourcesFormatter()
         assert (
             resource_formatter(resources) == "Resources:\n"
-            "  Memory: 16M\n"
+            "  Memory: 16.0M\n"
             "  CPU: 0.1"
         )
 
@@ -1041,7 +1391,7 @@ class TestResourcesFormatter:
         resource_formatter = ResourcesFormatter()
         assert (
             resource_formatter(resources) == "Resources:\n"
-            "  Memory: 1G\n"
+            "  Memory: 1.0G\n"
             "  CPU: 2.0\n"
             "  GPU: 1.0 x nvidia-tesla-p4"
         )
@@ -1059,7 +1409,7 @@ class TestResourcesFormatter:
         resource_formatter = ResourcesFormatter()
         assert (
             resource_formatter(resources) == "Resources:\n"
-            "  Memory: 16M\n"
+            "  Memory: 16.0M\n"
             "  CPU: 0.1\n"
             "  Additional: Extended SHM space"
         )
@@ -1077,7 +1427,7 @@ class TestResourcesFormatter:
         resource_formatter = ResourcesFormatter()
         assert (
             resource_formatter(resources=resources) == "Resources:\n"
-            "  Memory: 16M\n"
+            "  Memory: 16.0M\n"
             "  CPU: 0.1\n"
             "  TPU: v2-8/1.14\n"
             "  Additional: Extended SHM space"
