@@ -3,6 +3,7 @@ import hashlib
 import logging
 import os
 import re
+import shlex
 import subprocess
 import sys
 from collections import namedtuple
@@ -31,6 +32,7 @@ import pytest
 from yarl import URL
 
 from neuromation.api import (
+    AuthorizationError,
     Config,
     Container,
     FileStatusType,
@@ -96,7 +98,7 @@ def run_async(coro: Any) -> Callable[..., Any]:
 
 
 class Helper:
-    def __init__(self, nmrc_path: Path, tmp_path: Path) -> None:
+    def __init__(self, nmrc_path: Optional[Path], tmp_path: Path) -> None:
         self._nmrc_path = nmrc_path
         self._tmp = tmp_path
         self.tmpstoragename = f"test_e2e/{uuid()}"
@@ -190,8 +192,7 @@ class Helper:
         __tracebackhide__ = True
         url = self.make_uri(path, fromhome=fromhome)
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
-            files = await client.storage.ls(url)
-            for file in files:
+            async for file in client.storage.ls(url):
                 if (
                     file.type == FileStatusType.FILE
                     and file.name == name
@@ -205,8 +206,7 @@ class Helper:
         __tracebackhide__ = True
         url = URL(self.tmpstorage + path)
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
-            files = await client.storage.ls(url)
-            for file in files:
+            async for file in client.storage.ls(url):
                 if file.type == FileStatusType.DIRECTORY and file.path == name:
                     return
         raise AssertionError(f"Dir {name} not found in {url}")
@@ -216,8 +216,7 @@ class Helper:
         __tracebackhide__ = True
         url = URL(self.tmpstorage + path)
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
-            files = await client.storage.ls(url)
-            for file in files:
+            async for file in client.storage.ls(url):
                 if file.type == FileStatusType.DIRECTORY and file.path == name:
                     raise AssertionError(f"Dir {name} found in {url}")
 
@@ -226,8 +225,7 @@ class Helper:
         __tracebackhide__ = True
         url = URL(self.tmpstorage + path)
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
-            files = await client.storage.ls(url)
-            for file in files:
+            async for file in client.storage.ls(url):
                 if file.type == FileStatusType.FILE and file.path == name:
                     raise AssertionError(f"File {name} found in {url}")
 
@@ -282,12 +280,16 @@ class Helper:
                 URL(f"{self.tmpstorage}{path_from}/{name_from}"),
                 URL(f"{self.tmpstorage}{path_to}/{name_to}"),
             )
-            files1 = await client.storage.ls(URL(f"{self.tmpstorage}{path_from}"))
-            names1 = {f.name for f in files1}
+            names1 = {
+                f.name
+                async for f in client.storage.ls(URL(f"{self.tmpstorage}{path_from}"))
+            }
             assert name_from not in names1
 
-            files2 = await client.storage.ls(URL(f"{self.tmpstorage}{path_to}"))
-            names2 = {f.name for f in files2}
+            names2 = {
+                f.name
+                async for f in client.storage.ls(URL(f"{self.tmpstorage}{path_to}"))
+            }
             assert name_to in names2
 
     @run_async
@@ -461,6 +463,42 @@ class Helper:
             print(f"nero stderr: {err}")
         return SysCap(out, err)
 
+    def autocomplete(
+        self, arguments: List[str], *, network_timeout: float = NETWORK_TIMEOUT,
+    ) -> str:
+        __tracebackhide__ = True
+
+        log.info("Run 'neuro %s'", " ".join(arguments))
+
+        args = [
+            "neuro",
+            "--show-traceback",
+            "--disable-pypi-version-check",
+            "--color=no",
+            f"--network-timeout={network_timeout}",
+            "--skip-stats",
+        ]
+
+        if self._nmrc_path:
+            args.append(f"--neuromation-config={self._nmrc_path}")
+
+        env = dict(os.environ)
+        env["_NEURO_COMPLETE"] = "complete_zsh"
+        env["COMP_WORDS"] = " ".join(shlex.quote(arg) for arg in args + arguments)
+        env["COMP_CWORD"] = str(len(args + arguments) - 1)
+
+        proc = subprocess.run(
+            "neuro",
+            timeout=300,
+            encoding="utf8",
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            env=env,
+        )
+        assert proc.returncode == 1
+        assert not proc.stderr
+        return proc.stdout
+
     @run_async
     async def run_job_and_wait_state(
         self,
@@ -538,10 +576,71 @@ class Helper:
                         if stat.status not in (JobStatus.PENDING, JobStatus.RUNNING):
                             break
 
+    @run_async
+    async def create_bucket(self, name: str) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            await client.blob_storage.create_bucket(name)
+
+    @run_async
+    async def delete_bucket(self, name: str) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            await client.blob_storage.delete_bucket(name)
+
+    @run_async
+    async def cleanup_bucket(self, bucket_name: str) -> None:
+        __tracebackhide__ = True
+        # Each test needs a clean bucket state and we can't delete bucket until it's
+        # cleaned
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            blobs, _ = await client.blob_storage.list_blobs(bucket_name, recursive=True)
+            if not blobs:
+                return
+
+            # XXX: We do assume we will not have tests that run 10000 of objects. If we
+            # do, please add a semaphore here.
+            tasks = []
+            for blob in blobs:
+                log.info("Removing %s %s", bucket_name, blob.key)
+                tasks.append(client.blob_storage.delete_blob(bucket_name, key=blob.key))
+            await asyncio.gather(*tasks)
+
+    @run_async
+    async def upload_blob(self, bucket_name: str, key: str, file: Path) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            await client.blob_storage.upload_file(
+                URL("file:" + str(file)), client.blob_storage.make_url(bucket_name, key)
+            )
+
+    @run_async
+    async def check_blob_size(self, bucket_name: str, key: str, size: int) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            blob = await client.blob_storage.head_blob(bucket_name, key)
+            assert blob.size == size
+
+    @run_async
+    async def check_blob_checksum(
+        self, bucket_name: str, key: str, checksum: str, tmp_path: Path
+    ) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            await client.blob_storage.download_file(
+                client.blob_storage.make_url(bucket_name, key),
+                URL("file:" + str(tmp_path)),
+            )
+            assert self.hash_hex(tmp_path) == checksum, "checksum test failed for {url}"
+
 
 @pytest.fixture
 def nmrc_path(tmp_path_factory: Any, request: Any) -> Optional[Path]:
     require_admin = request.keywords.get("require_admin", False)
+    return _get_nmrc_path(tmp_path_factory, require_admin)
+
+
+def _get_nmrc_path(tmp_path_factory: Any, require_admin: bool) -> Optional[Path]:
     if require_admin:
         token_env = "E2E_TOKEN"
     else:
@@ -560,6 +659,9 @@ def nmrc_path(tmp_path_factory: Any, request: Any) -> Optional[Path]:
         )
         return nmrc_path
     else:
+        # By providing `None` we allow Helper to login using default configuration
+        # in user's home folder. Tests will use current logged in user and current
+        # cluster from neuro cli.
         return None
 
 
@@ -616,6 +718,31 @@ def nested_data(static_path: Path) -> Tuple[str, str, str]:
     return generated_file, hash, str(root_dir)
 
 
+@pytest.fixture(scope="session")
+def _tmp_bucket_create(
+    tmp_path_factory: Any, request: Any
+) -> Iterator[Tuple[str, Helper]]:
+    tmp_path = tmp_path_factory.mktemp("tmp_bucket" + str(uuid()))
+    tmpbucketname = f"neuro_test_e2e_{uuid()}"
+    nmrc_path = _get_nmrc_path(tmp_path_factory, require_admin=True)
+
+    helper = Helper(nmrc_path, tmp_path)
+    try:
+        helper.create_bucket(tmpbucketname)
+    except AuthorizationError:
+        pytest.skip("No permission to create bucket for user E2E_TOKEN")
+    yield tmpbucketname, helper
+    helper.delete_bucket(tmpbucketname)
+    helper.close()
+
+
+@pytest.fixture
+def tmp_bucket(_tmp_bucket_create: Tuple[str, Helper]) -> Iterator[str]:
+    tmpbucketname, helper = _tmp_bucket_create
+    yield tmpbucketname
+    helper.cleanup_bucket(tmpbucketname)
+
+
 @pytest.fixture
 def secret_job(helper: Helper) -> Callable[[bool, bool, Optional[str]], Dict[str, Any]]:
     def go(
