@@ -2,9 +2,11 @@ import asyncio
 import os
 import re
 import subprocess
+import sys
 from contextlib import suppress
+from datetime import datetime, timedelta
 from pathlib import Path
-from time import time
+from time import sleep, time
 from typing import Any, AsyncIterator, Callable, Dict, List, Tuple
 from uuid import uuid4
 
@@ -63,6 +65,8 @@ def test_job_submit(helper: Helper) -> None:
             "80",
             "--non-preemptible",
             "--no-wait-start",
+            "--restart",
+            "never",
             "--name",
             job_name,
             UBUNTU_IMAGE_NAME,
@@ -194,6 +198,46 @@ def test_job_tags(helper: Helper) -> None:
     assert set(tags) <= set(tags_listed)
 
 
+@pytest.mark.e2e
+def test_job_filter_by_date_range(helper: Helper) -> None:
+    captured = helper.run_cli(
+        [
+            "job",
+            "run",
+            "-s",
+            JOB_TINY_CONTAINER_PRESET,
+            "--no-wait-start",
+            UBUNTU_IMAGE_NAME,
+            "sleep 300",
+        ]
+    )
+    match = re.match("Job ID: (.+) Status:", captured.out)
+    assert match is not None
+    job_id = match.group(1)
+    now = datetime.now()
+    delta = timedelta(minutes=10)
+
+    captured = helper.run_cli(["ps", "--since", (now - delta).isoformat()])
+    store_out_list = captured.out.split("\n")[1:]
+    jobs = [x.split("  ")[0] for x in store_out_list]
+    assert job_id in jobs
+
+    captured = helper.run_cli(["ps", "--since", (now + delta).isoformat()])
+    store_out_list = captured.out.split("\n")[1:]
+    jobs = [x.split("  ")[0] for x in store_out_list]
+    assert job_id not in jobs
+
+    captured = helper.run_cli(["ps", "--until", (now - delta).isoformat()])
+    store_out_list = captured.out.split("\n")[1:]
+    jobs = [x.split("  ")[0] for x in store_out_list]
+    assert job_id not in jobs
+
+    captured = helper.run_cli(["ps", "--until", (now + delta).isoformat()])
+    store_out_list = captured.out.split("\n")[1:]
+    jobs = [x.split("  ")[0] for x in store_out_list]
+    assert job_id in jobs
+
+
 @pytest.mark.e2e
 def test_job_kill_non_existing(helper: Helper) -> None:
     # try to kill non existing job
@@ -573,7 +617,7 @@ async def nginx_job_async(
             f"timeout 15m /usr/sbin/nginx -g 'daemon off;'\""
         )
         container = Container(
-            image=RemoteImage("nginx", "latest"),
+            image=RemoteImage.new_external_image(name="nginx", tag="latest"),
             command=command,
             resources=Resources(20, 0.1, None, None, True, None, None),
         )
@@ -883,7 +927,7 @@ def test_job_run_no_detach_browse_failure(helper: Helper) -> None:
             ]
         )
     assert captured is None
-    assert exc_info.value.returncode == 125
+    assert exc_info.value.returncode == 127
 
 
 @pytest.mark.e2e
@@ -907,6 +951,9 @@ def test_job_run_with_tty(helper: Helper) -> None:
     # Wait until the job is running
     helper.wait_job_change_state_to(job_id, JobStatus.SUCCEEDED)
 
+    captured = helper.run_cli(["job", "status", job_id])
+    assert "TTY: True" in captured.out
+
 
 @pytest.mark.e2e
 def test_job_run_home_volumes_automount(helper: Helper, fakebrowser: Any) -> None:
@@ -915,18 +962,10 @@ def test_job_run_home_volumes_automount(helper: Helper, fakebrowser: Any) -> Non
     with pytest.raises(subprocess.CalledProcessError) as cm:
         # first, run without --volume=HOME
         helper.run_cli(
-            [
-                "-q",
-                "job",
-                "run",
-                "--detach",
-                "--preset=cpu-micro",
-                UBUNTU_IMAGE_NAME,
-                command,
-            ]
+            ["-q", "job", "run", "--preset=cpu-micro", UBUNTU_IMAGE_NAME, command]
         )
 
-    assert cm.value.returncode == 125
+    assert cm.value.returncode == 1
 
     # then, run with --volume=HOME
     capture = helper.run_cli(
@@ -934,7 +973,6 @@ def test_job_run_home_volumes_automount(helper: Helper, fakebrowser: Any) -> Non
             "-q",
             "job",
             "run",
-            "--detach",
             "--preset=cpu-micro",
             "--volume",
             "HOME",
@@ -964,15 +1002,11 @@ def test_job_run_volume_all(helper: Helper) -> None:
 
     with pytest.raises(subprocess.CalledProcessError) as cm:
         # first, run without --volume=ALL
-        captured = helper.run_cli(
-            ["--quiet", "run", "--detach", "-s", "cpu-micro", img, command]
-        )
-    assert cm.value.returncode == 125
+        captured = helper.run_cli(["--quiet", "run", "-s", "cpu-micro", img, command])
+    assert cm.value.returncode == 1
 
     # then, run with --volume=ALL
-    captured = helper.run_cli(
-        ["run", "--detach", "-s", "cpu-micro", "--volume=ALL", img, command]
-    )
+    captured = helper.run_cli(["run", "-s", "cpu-micro", "--volume=ALL", img, command])
     msg = (
         "Storage mountpoints will be available as the environment variables:\n"
         f"  NEUROMATION_ROOT={root_mountpoint}\n"
@@ -1082,3 +1116,65 @@ def test_e2e_job_top(helper: Helper) -> None:
         ]
         for actual, (descr, pattern) in zip(line_parts, expected_parts):
             assert re.match(pattern, actual) is not None, f"error in matching {descr}"
+
+
+@pytest.mark.e2e
+def test_e2e_restart_failing(request: Any, helper: Helper) -> None:
+    captured = helper.run_cli(
+        [
+            "-q",
+            "job",
+            "run",
+            "--restart",
+            "on-failure",
+            "-s",
+            JOB_TINY_CONTAINER_PRESET,
+            "--detach",
+            UBUNTU_IMAGE_NAME,
+            "false",
+        ]
+    )
+    job_id = captured.out
+    request.addfinalizer(lambda: helper.kill_job(job_id, wait=False))
+
+    captured = helper.run_cli(["job", "status", job_id])
+    assert "Restart policy: on-failure" in captured.out.splitlines()
+
+    helper.wait_job_change_state_to(job_id, JobStatus.RUNNING)
+    sleep(1)
+    helper.assert_job_state(job_id, JobStatus.RUNNING)
+
+
+@pytest.mark.skipif(
+    sys.platform == "win32", reason="Autocompletion is not supported on Windows"
+)
+@pytest.mark.e2e
+def test_job_autocomplete(helper: Helper) -> None:
+
+    job_name = f"test-job-{os.urandom(5).hex()}"
+    helper.kill_job(job_name)
+    job_id = helper.run_job_and_wait_state(ALPINE_IMAGE_NAME, "sleep 60", name=job_name)
+
+    out = helper.autocomplete(["kill", "test-job"])
+    assert job_name in out
+    assert job_id not in out
+
+    out = helper.autocomplete(["kill", "job-"])
+    assert job_name in out
+    assert job_id in out
+
+    out = helper.autocomplete(["kill", "job:job-"])
+    assert job_name in out
+    assert job_id in out
+
+    out = helper.autocomplete(["kill", f"job:/{helper.username}/job-"])
+    assert job_name in out
+    assert job_id in out
+
+    out = helper.autocomplete(
+        ["kill", f"job://{helper.cluster_name}/{helper.username}/job-"]
+    )
+    assert job_name in out
+    assert job_id in out
+
+    helper.kill_job(job_id)
